@@ -66,8 +66,7 @@ struct CBUF_Header *rpc2_TraceBuffHeader = NULL;
 PROCESS rpc2_SocketListenerPID=NULL;
 
 long Retry_N;			/* total number of retries -- see packet.c */
-struct timeval *Retry_Beta;	/* array of timeout intervals */
-struct timeval SaveResponse;    /* 2*Beta0: lifetime of saved response packet */
+struct timeval KeepAlive;	/* keepalive timeout */
 long rpc2_Bandwidth = 10485760; /* bandwidth hint supplied externally */
 
 /* Doubly-linked lists and counts */
@@ -112,7 +111,7 @@ long rpc2_AllocMgrps, rpc2_FreeMgrps;
 
 long rpc2_HoldHWMark, rpc2_FreezeHWMark;
 
-char *rpc2_LastEdit = "$Header: /coda-src/rpc2/rpc2-src/globals.c,v 4.11 2003-11-13 23:06:52 jaharkes Exp $";
+char *rpc2_LastEdit = "$Header: /coda-src/rpc2/rpc2-src/globals.c,v 4.45 2006-10-13 03:04:17 jaharkes Exp $";
 
 long rpc2_errno;
 
