@@ -1,26 +1,26 @@
 /* BLURB lgpl
 
-                           Coda File System
-                              Release 5
+			Coda File System
+			    Release 6
 
-          Copyright (c) 1987-1999 Carnegie Mellon University
-                  Additional copyrights listed below
+	    Copyright (c) 1987-2006 Carnegie Mellon University
+		    Additional copyrights listed below
 
 This  code  is  distributed "AS IS" without warranty of any kind under
 the  terms of the  GNU  Library General Public Licence  Version 2,  as
 shown in the file LICENSE. The technical and financial contributors to
 Coda are listed in the file CREDITS.
 
-                        Additional copyrights
+			Additional copyrights
 
 #*/
 
 /*
-                         IBM COPYRIGHT NOTICE
+			IBM COPYRIGHT NOTICE
 
-                          Copyright (C) 1986
-             International Business Machines Corporation
-                         All Rights Reserved
+			  Copyright (C) 1986
+	       International Business Machines Corporation
+			  All Rights Reserved
 
 This  file  contains  some  code identical to or derived from the 1986
 version of the Andrew File System ("AFS"), which is owned by  the  IBM
@@ -49,7 +49,7 @@ Pittsburgh, PA.
 #include <string.h>
 #include "rpc2.private.h"
 
-/* Code to track host liveness 
+/* Code to track host liveness
 
    We track liveness by host ip-address, as we're really interested
    in the bandwidth and latency of connection between us and the server
@@ -62,7 +62,7 @@ Pittsburgh, PA.
    connections, because of the possibility of lost request packets.
    Hosts are kept in a hash table.
    We hash on the low bytes of the host address (in network order) */
- 
+
 #define HOSTHASHBUCKETS 64
 /* try to grab the low-order bits, assuming all are stored big endian */
 int HASHHOST(struct RPC2_addrinfo *ai)
@@ -142,10 +142,9 @@ struct HEntry *rpc2_GetHost(struct RPC2_addrinfo *addr)
     rpc2_ClearHostLog(he, RPC2_MEASUREMENT);
     rpc2_ClearHostLog(he, SE_MEASUREMENT);
 
-    he->RTT       = 0;
-    he->RTTVar    = 0;
-    he->BR        = 1000 << RPC2_BR_SHIFT;
-    he->BRVar     = 0;
+    he->RTT = 0;
+    he->BWlo_out = he->BWhi_out = RPC2_INITIAL_BW;
+    he->BWlo_in = he->BWhi_in = RPC2_INITIAL_BW;
 
     /* insert into hash table */
     he->HLink = HostHashTable[bucket];
@@ -197,6 +196,9 @@ void rpc2_FreeHost(struct HEntry **whichHost)
 		}
 		link = &(*link)->HLink;
 	}
+
+	LUA_drop_hosttable(*whichHost);
+
 	*whichHost = NULL;
 }
 
@@ -308,139 +310,177 @@ void rpc2_ClearHostLog(struct HEntry *whichHost, NetLogEntryType type)
 	}
 }
 
-/* Here we update the RTT and Bandwidth (or more precise byterate in ns per
- * byte) estimates,
+static void getestimates(struct HEntry *host, uint32_t outb, uint32_t inb,
+			 uint32_t *rtt_lat, uint32_t *rtt_in, uint32_t *rtt_out)
+{
+    uint32_t avgBW;
+
+    *rtt_lat = host->RTT >> RPC2_RTT_SHIFT;
+
+    avgBW = (host->BWlo_in >> 1) + (host->BWhi_in >> 1);
+    while (inb > 2048) { inb >>= 1; avgBW >>= 1; }
+    if (!avgBW) avgBW = 1;
+    *rtt_in = (1000000 * inb) / avgBW;
+
+    avgBW = (host->BWlo_out >> 1) + (host->BWhi_out >> 1);
+    while (outb > 2048) { outb >>= 1; avgBW >>= 1; }
+    if (!avgBW) avgBW = 1;
+    *rtt_out = (1000000 * outb) / avgBW;
+}
+
+static void update_bw(uint32_t *BWlo, uint32_t *BWhi, uint32_t rtt,
+		      uint32_t bytes)
+{
+    uint32_t current, estimated;
+
+    /* eBR = ( eU * 1000 ) / Bytes ; */
+    current = (rtt * 125 / bytes) << 3;
+
+    if (!*BWlo) *BWlo = 1;
+    estimated = 1000000000 / *BWlo;
+    if (current >= estimated)
+	 estimated += (current - estimated) >> RPC2_BW_SHIFT;
+    else estimated -= (estimated - current) >> RPC2_BW_SHIFT;
+    if (!estimated) estimated = 1;
+    *BWlo = 1000000000 / estimated;
+
+    while (bytes > 4096) { bytes >>= 1; rtt >>= 1; }
+    if (!rtt) rtt = 1;
+    current = 1000000 * bytes / rtt;
+    if (current >= *BWhi)
+	 *BWhi += (current - *BWhi) >> RPC2_BW_SHIFT;
+    else *BWhi -= (*BWhi - current) >> RPC2_BW_SHIFT;
+}
+
+/* Here we update the RTT and Bandwidth estimates,
  * ElapsedTime  is the observed roundtrip-time in microseconds
- * Bytes        is the number of bytes transferred */
+ * Bytes	is the number of bytes transferred */
 void RPC2_UpdateEstimates(struct HEntry *host, RPC2_Unsigned elapsed_us,
 			  RPC2_Unsigned InBytes, RPC2_Unsigned OutBytes)
 {
-    long	   eRTT;     /* estimated null roundtrip time */
-    long	   eBR;      /* estimated byterate (ns/B) */
-    unsigned long  eU;       /* temporary unsigned variable */
-    long	   eL;       /* temporary signed variable */
-    char addr[RPC2_ADDRSTRLEN];
+    uint32_t rto, rtt_lat, rtt_out, rtt_in;/* estimated roundtrip time(s) */
+    uint32_t rttvar = host->RTTvar >> RPC2_RTTVAR_SHIFT;
+    int32_t adjustment;
 
     if (!host) return;
 
-    say(0, RPC2_DebugLevel, "uRTT: %u %u %u\n", elapsed_us, InBytes, OutBytes);
-    
+    /* account for IP/UDP header overhead
+     * an IPv4 header is (typically) 20 bytes but can be up to 60 bytes, IPv6
+     * headers are 40 bytes but could include additional headers such as an 8
+     * byte fragment header. In addition the UDP header adds another 8 bytes.
+     * And then there is an additional 18 bytes ethernet header, but that may
+     * not exist on PPP links. And of course if we have some compression layer
+     * below us none of these numbers make any sense.
+     * If we pick 40 we will be slightly reasonably close for a IPv4 ethernet
+     * network but underestimate the packet size over a v6 network. At least it
+     * brings us a bit closer to reality. If we don't account for this
+     * overhead, the delay of a 60 byte RPC2 ping packet is considerably
+     * underestimated, which leads to an incorrect bandwidth estimate. */
+    InBytes += 40; OutBytes += 40;
     if ((int32_t)elapsed_us < 0) elapsed_us = 0;
 
-    /* we need to clamp elapsed elapsed_us to about 16 seconds to avoid
-     * overflows with the 31 bit calculations below */
-    if (elapsed_us > 0x00ffffff) elapsed_us = 0x00ffffff;
+    getestimates(host, OutBytes, InBytes, &rtt_lat, &rtt_in, &rtt_out);
+    rto = rtt_lat + rtt_out + rtt_in;
 
-    /* get the estimated rtt */
-    eRTT = host->RTT >> RPC2_RTT_SHIFT;
+    if (RPC2_DebugLevel)
+    {
+	char addr[RPC2_ADDRSTRLEN];
+	RPC2_formataddrinfo(host->Addr, addr, RPC2_ADDRSTRLEN);
+	fprintf(rpc2_logfile, "uRTT: %s %u %u %u %u %u %u %u %u %u\n",
+		addr, elapsed_us, OutBytes, InBytes, rto, rtt_lat,
+		host->BWlo_out, host->BWhi_out, host->BWlo_in, host->BWhi_in);
+    }
 
-    if (elapsed_us > eRTT) eU = elapsed_us - eRTT;
-    else		   eU = 0;
+    if (elapsed_us >= rto) {
+	adjustment = (elapsed_us - rto) / 3;
+	rtt_out += adjustment;
+	rtt_in += adjustment;
+    } else {
+	adjustment = elapsed_us / 3;
+	rtt_out = adjustment;
+	rtt_in = adjustment;
+	adjustment -= rtt_lat;
+    }
 
-    /* eBR = ( eU * 1000 ) / Bytes ; */
-    eBR = ((eU << 7) / (InBytes + OutBytes)) << 3;
-    eBR -= (host->BR >> RPC2_BR_SHIFT);
+    host->RTT += adjustment;
+    update_bw(&host->BWlo_in, &host->BWhi_in, rtt_in, InBytes);
+    update_bw(&host->BWlo_out, &host->BWhi_out, rtt_out, OutBytes);
 
-    /* HACK! to avoid small packets with RTT's that are not within the current
-     * window of variance to have a strong effect on the byterate estimate, we
-     * halve the difference of the new measurement, if it falls outside of the
-     * variance window */
-    if      (eBR >  (host->BRVar >> RPC2_BRVAR_SHIFT)) eBR >>= 1;
-    else if (eBR < -(host->BRVar >> RPC2_BRVAR_SHIFT)) eBR >>= 1;
-    /* HACK! */
+    if (adjustment < 0) adjustment = -adjustment;
+    if (adjustment >= rttvar)
+	 host->RTTvar += adjustment - rttvar;
+    else host->RTTvar -= rttvar - adjustment;
 
-    host->BR += eBR;
-
-    if (eBR < 0) eBR = -eBR;
-
-    /* Invariant: eBR contains the absolute difference between the previous
-     * calculated byterate and the new measurement */
-
-    eBR -= (host->BRVar >> RPC2_BRVAR_SHIFT);
-    host->BRVar += eBR;
-
-    /* get a new RTT estimate in elapsed_us */
-    /* from here on eBR contains a lower estimate on the effective
-     * byterate, eRTT will contain a updated RTT estimate */
-    eBR = (host->BR >> RPC2_BR_SHIFT) + ((host->BR >> RPC2_BRVAR_SHIFT) >> 1);
-
-    /* eU = ( eBR * (InBytes + OutBytes ) / 1000 ; */
-    eU = ((eBR >> 4) * (InBytes + OutBytes)) >> 6;
-
-    if (elapsed_us > eU) eL = elapsed_us - eU;
-    else		 eL = 0;
-
-    /* the RTT & RTT variance are shifted analogous to Jacobson's
-     * article in SIGCOMM'88 */
-    eL -= (host->RTT >> RPC2_RTT_SHIFT);
-    host->RTT += eL;
-
-    if (eL < 0) eL = -eL;
-
-    eL -= (host->RTTVar >> RPC2_RTTVAR_SHIFT);
-    host->RTTVar += eL;
-
-    RPC2_formataddrinfo(host->Addr, addr, RPC2_ADDRSTRLEN);
-    say(0, RPC2_DebugLevel,
-	"Est: %s %4d.%06u/%-5u<%-5u RTT:%lu/%lu us BR:%lu/%lu ns/B\n",
-	    addr, elapsed_us / 1000000, elapsed_us % 1000000,
-	    InBytes, OutBytes, host->RTT>>RPC2_RTT_SHIFT, host->RTTVar>>RPC2_RTTVAR_SHIFT,
-	    (host->BR>>RPC2_BR_SHIFT), host->BRVar>>RPC2_BRVAR_SHIFT);
-    return;
+    LUA_rtt_update(host, elapsed_us, OutBytes, InBytes);
 }
 
-void rpc2_RetryInterval(RPC2_Handle whichConn, RPC2_Unsigned InBytes,
-			RPC2_Unsigned OutBytes, int *retry,
-			int maxretry, struct timeval *tv)
+static uint32_t rpc2_GetRTO(struct HEntry *he, uint32_t outb, uint32_t inb)
 {
-    struct CEntry *ce;
-    unsigned long rto, rtt;
-    long          effBR;
-    int		  i;
+    uint32_t rto, rtt_lat, rtt_in, rtt_out, rttvar;
 
-    ce = rpc2_GetConn(whichConn);
-
-    if (!ce || !tv) {
-	say(0, RPC2_DebugLevel, "RetryInterval: !ce || !tv\n");
-	return;
+    rto = LUA_rtt_getrto(he, outb, inb);
+    if ((int32_t)rto <= 0) {
+	rttvar = he->RTTvar >> RPC2_RTTVAR_SHIFT;
+	getestimates(he, inb, outb, &rtt_lat, &rtt_in, &rtt_out);
+	rto = rtt_lat + rtt_out + rtt_in + (rttvar << 1);
+	say(4, RPC2_DebugLevel,
+	    "rpc2_GetRTO: rto %u, lat %u, out %u, in %u, var %u\n",
+	    rto, rtt_lat, rtt_out, rtt_in, rttvar);
     }
+    return rto;
+}
+
+int rpc2_RetryInterval(struct CEntry *ce, int retry, struct timeval *tv,
+		       RPC2_Unsigned OutBytes, RPC2_Unsigned InBytes)
+{
+    uint32_t rto, maxrtt;
+    int i = 0;
+
+    if (!ce) {
+	say(1, RPC2_DebugLevel, "RetryInterval: !conn\n");
+	return -1;
+    }
+
+    /* Account for IP/UDP header overhead, see rpc2_UpdateEstimates */
+    InBytes += 40; OutBytes += 40;
 
     /* calculate the estimated RTT */
+    rto = LUA_rtt_retryinterval(ce->HostInfo, retry, OutBytes, InBytes);
+    if (rto == 0) {
+	rto = rpc2_GetRTO(ce->HostInfo, OutBytes, InBytes);
 
-    rto = (ce->HostInfo->RTT >> RPC2_RTT_SHIFT) + (ce->HostInfo->RTTVar >> 1);
+	if (retry) {
+	    maxrtt = ce->KeepAlive.tv_sec * 1000000 + ce->KeepAlive.tv_usec;
 
-    /* because we have subtracted the time to took to transfer data from our
-     * RTT estimate (it is latency estimate), we have to add in the time to
-     * send our packet into the estimated RTO */
-
-    effBR = (ce->HostInfo->BR >> RPC2_BR_SHIFT);
-
-    /* rto += ( effBR * (InBytes + OutBytes) ) / 1000 ; */
-    rto += ((effBR >> 3) * (InBytes + OutBytes)) >> 7;
-    
-    if (*retry != 1) {
-	rtt = ce->MaxRetryInterval.tv_sec * 1000000 +
-	      ce->MaxRetryInterval.tv_usec;
-
-	for (i = maxretry; i > *retry; i--) {
-	    rtt >>= 1;
-	    if (rtt < rto) break;
+	    for (i = Retry_N; i > 0; i--) {
+		maxrtt >>= 1;
+		if (rto > maxrtt) break;
+	    }
+	    rto = maxrtt;
 	}
-	*retry = i;
-	if (rtt > rto) rto = rtt;
+	if (i + retry > Retry_N) return -1;
+	rto <<= retry;
     }
-    
-    /* clamp retry estimates */
-    if      (rto < RPC2_MINRTO) rto = RPC2_MINRTO;
-    else if (rto > RPC2_MAXRTO) rto = RPC2_MAXRTO;
+
+    /* account for server processing overhead */
+    rto += RPC2_DELACK_DELAY;
+
+    /* clamp retry estimate */
+    /* we shouldn't need a lower bound because we already account for the
+     * server processing delay */
+    if (rto > RPC2_MAXRTO) rto = RPC2_MAXRTO;
 
     tv->tv_sec  = rto / 1000000;
     tv->tv_usec = rto % 1000000;
 
-    say(0, RPC2_DebugLevel, "RetryInterval: %lu.%06lu\n",
-	tv->tv_sec, tv->tv_usec);
-
-    return;
+    if (RPC2_DebugLevel > 1)
+    {
+	char addr[RPC2_ADDRSTRLEN];
+	RPC2_formataddrinfo(ce->HostInfo->Addr, addr, RPC2_ADDRSTRLEN);
+	fprintf(rpc2_logfile, "RetryInterval: %s %d %d %ld.%06lu\n",
+		addr, retry, i, tv->tv_sec, tv->tv_usec);
+    }
+    return 0;
 }
 
 int RPC2_GetRTT(RPC2_Handle handle, unsigned long *RTT, unsigned long *RTTvar)
@@ -448,11 +488,11 @@ int RPC2_GetRTT(RPC2_Handle handle, unsigned long *RTT, unsigned long *RTTvar)
     struct CEntry *ce;
 
     ce = rpc2_GetConn(handle);
-    if (ce == NULL) 
+    if (ce == NULL)
 	return(RPC2_NOCONNECTION);
 
     if (RTT)    *RTT    = ce->HostInfo->RTT    >> RPC2_RTT_SHIFT;
-    if (RTTvar) *RTTvar = ce->HostInfo->RTTVar >> RPC2_RTTVAR_SHIFT;
+    if (RTTvar) *RTTvar = ce->HostInfo->RTTvar >> RPC2_RTTVAR_SHIFT;
 
     return(RPC2_SUCCESS);
 }
@@ -461,36 +501,23 @@ int RPC2_GetBandwidth(RPC2_Handle handle, unsigned long *BWlow,
 		      unsigned long *BWavg, unsigned long *BWhigh)
 {
     struct CEntry *ce;
-    unsigned long BR, BRVar, tBR;
+    uint32_t bw_lo, bw_avg, bw_hi;
 
     ce = rpc2_GetConn(handle);
     if (ce == NULL) return(RPC2_NOCONNECTION);
 
-    BR    = ce->HostInfo->BR    >> RPC2_BR_SHIFT;
-    BRVar = ce->HostInfo->BRVar >> RPC2_BRVAR_SHIFT;
-
-    if (BWlow) {
-	tBR = BR + BRVar;
-	/* Need at least 1 to avoid divide by zero errors --JH */
-	if (!tBR) tBR = 1;
-
-	*BWlow = 1000000000 / tBR;
+    if (LUA_rtt_getbandwidth(ce->HostInfo, &bw_avg, NULL))
+	bw_lo = bw_hi = bw_avg;
+    else
+    {
+	bw_lo = ce->HostInfo->BWlo_out;
+	bw_avg = (ce->HostInfo->BWlo_out + ce->HostInfo->BWhi_out) >> 1;
+	bw_hi = ce->HostInfo->BWhi_out;
     }
-    if (BWavg) {
-	tBR = BR;
-	/* Need at least 1 to avoid divide by zero errors --JH */
-	if (!tBR) tBR = 1;
 
-	*BWavg = 1000000000 / tBR;
-    }
-    if (BWhigh) {
-	if (BR > BRVar)
-	    tBR = BR - BRVar;
-	else
-	    tBR = 1;
-
-	*BWhigh = 1000000000 / tBR;
-    }
+    if (BWlow)  *BWlow = bw_lo;
+    if (BWavg)  *BWavg = bw_avg;
+    if (BWhigh) *BWhigh = bw_hi;
 
     return(RPC2_SUCCESS);
 }
@@ -500,7 +527,7 @@ int RPC2_GetLastObs(RPC2_Handle handle, struct timeval *tv)
     struct CEntry *ce;
 
     ce = rpc2_GetConn(handle);
-    if (ce == NULL) 
+    if (ce == NULL)
 	return(RPC2_NOCONNECTION);
 
     if (tv) *tv = ce->HostInfo->LastWord;
