@@ -133,9 +133,11 @@ OBJ_HENTRY = 48127
 #define O_BINARY 0
 #endif
 
-/* MINRTO/MAXRTO are used to avoid unbounded timeouts */
-#define RPC2_MINRTO   300000        /* min rto (rtt + variance) is 300 msec */
-#define RPC2_MAXRTO   30000000     /* max rto (rtt + variance) is 30 seconds */
+/* MAXRTO is used to avoid unbounded timeouts */
+#define RPC2_MAXRTO     30000000   /* max rto (rtt + variance) is 30 seconds */
+#define RPC2_DELACK_DELAY 100000   /* delay for server to send an ack that it
+				      received a request. This provides an
+				      upper bound on server response time. */
 
 /* Definitions for Flags field of connections */
 #define CE_OLDV  0x1  /* old version detected during bind */
@@ -194,19 +196,11 @@ struct CEntry		/* describes a single RPC connection */
 					    for retransmission (reply or last packet
 					    of Bind handshake */
     unsigned long reqsize;              /* size of request, for RTT calcs */
-    unsigned long respsize;              /* size of response, for RTT calcs */
 
-    /* Retransmission info - client */
-#define LOWERLIMIT 300000               /* floor on lower limit, usec. */
-    unsigned long LowerLimit;           /* minimum retry interval, usec */
-
-    long RTT;                           /* Smoothed RTT estimate, 1msec units. */
-    long RTTVar;                        /* Variance of RTT, 1msec units. */
     unsigned int TimeStampEcho;
     unsigned int RequestTime;
-    long Retry_N;                       /* Number of retries for this connection. */
-    struct timeval *Retry_Beta;         /* Retry parameters for this connection. */
-    struct timeval SaveResponse;        /* 2*Beta0, lifetime of saved response packet. */
+    struct timeval KeepAlive;           /* Retry parameter for this connection. */
+    struct timeval SaveResponse;        /* 2*KeepAlive, lifetime of saved response packet. */
     RPC2_RequestFilter Filter;		/* Set on the server during binding,
 					   filter incoming requests so that the
 					   SubsysID/Connection matches that of
@@ -272,6 +266,9 @@ struct MEntry			/* describes an RPC multicast connection */
 		       destroyed by SocketListener on timeout
 	               destroyed by user LWP when reply is sent.
 
+          DELAYED_SEND Artificially introduced a transmission delay
+          DELAYED_RECV Artificially introduced a reception delay
+
           OTHER        associated with a specific connection
 	               created and destroyed by user LWP
 
@@ -282,7 +279,8 @@ connection they are associated with.
 */
 
 /* NOTE:  enum definitions  have to be non-anonymous: else a dbx bug is triggered */
-enum SL_Type {REPLY=1421, REQ=1422, OTHER=1423, DELACK=20010911};
+enum SL_Type {REPLY=1421, REQ=1422, OTHER=1423, DELACK=20010911,
+	      DELAYED_SEND=20061016, DELAYED_RECV=20061017};
 enum RetVal {WAITING=38358230, ARRIVED=38358231, TIMEOUT=38358232,
 	KEPTALIVE=38358233, KILLED=38358234, NAKED=38358235};
 
@@ -305,9 +303,10 @@ struct SL_Entry
 
     /* Other fields */
     RPC2_Handle Conn;		/* NULL or conn corr to this SL Entry */
-    RPC2_PacketBuffer *Packet;  /* NULL,  awaiting retransmission or just arrived */
+    void *data;			/* NULL, packet buffer, or delayed xmit data */
     RPC2_RequestFilter Filter;  /* useful only in GetRequest */
-    long RetryIndex;		/* useful only in MakeRPC */
+    int RetryIndex;		/* useful only in MakeRPC */
+    struct timeval RInterval;	/* current retry timeout */
     };
 
 
@@ -342,15 +341,16 @@ struct HEntry {
     unsigned SE_NumEntries;	/* number of sideeffect observations recorded */
     RPC2_NetLogEntry SE_Log[RPC2_MAXLOGLENGTH];
 
+/* the RTT is shifted analogous to Jacobson's article in SIGCOMM'88 */
 #define RPC2_RTT_SHIFT    3     /* Bits to right of binary point of RTT */
-#define RPC2_RTTVAR_SHIFT 2     /* Bits to right of binary point of RTTVar */
-    unsigned long   RTT;	/* RTT          (us<<RPC2_RTT_SHIFT) */
-    unsigned long   RTTVar;	/* RTT variance (us<<RPC2_RTTVAR_SHIFT) */
+#define RPC2_RTTVAR_SHIFT 2     /* Bits to right of binary point of RTTvar */
+    unsigned long   RTT;	/* RTT		(us<<RPC2_RTT_SHIFT) */
+    unsigned long   RTTvar;	/* RTT variance	(us<<RPC2_RTTVAR_SHIFT) */
 
-#define RPC2_BR_SHIFT    3
-#define RPC2_BRVAR_SHIFT 2
-    unsigned long   BR;		/* Byterate          (ns/B<<RPC2_BW_SHIFT) */
-    unsigned long   BRVar;	/* Byterate variance (ns/B<<RPC2_BWVAR_SHIFT) */
+#define RPC2_INITIAL_BW 100000
+#define RPC2_BW_SHIFT    4
+    uint32_t BWlo_in,  BWhi_in;	/* Incoming Bandwidth	(B/s) */
+    uint32_t BWlo_out, BWhi_out; /* Outgoing Bandwidth	(B/s) */
 };
 
 
@@ -359,8 +359,20 @@ struct HEntry {
 /* WARNING: if you port RPC2, make sure the structure sizes work out to be the same on your target machine */
 
 struct Init1Body			/* Client to Server: format of packets with opcode of RPC2_INIT1xxx */
-    {
-    RPC2_NewConnectionBody FakeBody;	/* body of fake packet from RPC2_GetRequest() */
+{
+/* body of fake packet from RPC2_GetRequest() */
+    RPC2_Integer FakeBody_SideEffectType;
+    RPC2_Integer FakeBody_SecurityLevel;
+    RPC2_Integer FakeBody_EncryptionType;
+    RPC2_Integer FakeBody_AuthenticationType;
+    RPC2_Unsigned FakeBody_ClientIdent_SeqLen;
+    RPC2_Unsigned FakeBody_ClientIdent_SeqBody;
+/* end of body of fake packet from RPC2_GetRequest() */
+    /* When MakeFake is called we'll clobber the remaining data in this packet
+     * because we don't really send a pointer to the ClientIdent.SeqBody, but
+     * we move it there from the tail of the packet so that the stub generator
+     * can unpack it as a valid RPC2_NEWCONNECTION rpc call */
+
     RPC2_Integer XRandom;		/* encrypted random number */
     char usedtobehostport[92];		/* XXX not used anymore but old rpc2
 					   servers need the alignment */
@@ -369,9 +381,11 @@ struct Init1Body			/* Client to Server: format of packets with opcode of RPC2_IN
     RPC2_Unsigned Preferred_Keysize;	/* preferred encryption key length */
     RPC2_Integer Spare3;
     RPC2_Byte Version[96];		/* set to RPC2_VERSION */
-    RPC2_Byte Text[4];	    /* Storage pointed to by FakeBody.ClientIdent.SeqBody;
-				4 bytes is a fake to cause proper longword alignment
-				on all interesting machines */
+    RPC2_Byte Text[4];			/* Storage for the ClientIdent. Moved
+					   to FakeBody_ClientIdent_SeqBody in
+					   rpc2a.c:MakeFake(). It can of course
+					   be more than 4 bytes, this is just
+					   for alignment purposes. */
     };
 
 struct Init2Body		/* Server to Client */
@@ -458,9 +472,7 @@ extern struct CBUF_Header *rpc2_TraceBuffHeader;
 extern PROCESS rpc2_SocketListenerPID;	/* used in IOMGR_Cancel() calls */
 extern unsigned long rpc2_LamportClock;
 extern long Retry_N;
-extern struct timeval *Retry_Beta;
-#define MaxRetryInterval Retry_Beta[0]
-extern struct timeval SaveResponse;
+extern struct timeval KeepAlive;
 
 /* List manipulation routines */
 void rpc2_Replenish();
@@ -486,10 +498,8 @@ int rpc2_MorePackets(void);
 long rpc2_RecvPacket(long whichSocket, RPC2_PacketBuffer *whichBuff);
 void rpc2_htonp(RPC2_PacketBuffer *p);
 void rpc2_ntohp(RPC2_PacketBuffer *p);
-long rpc2_SetRetry(), rpc2_CancelRetry();
-void rpc2_ResetLowerLimit();
+long rpc2_CancelRetry(struct CEntry *Conn, struct SL_Entry *Sle);
 void rpc2_UpdateRTT(RPC2_PacketBuffer *pb, struct CEntry *ceaddr);
-void rpc2_ResetObs();
 void rpc2_ExpireEvents();
 
 /* Connection manipulation routines  */
@@ -516,9 +526,8 @@ void rpc2_ClearHostLog(struct HEntry *whichHost, NetLogEntryType type);
 
 void RPC2_UpdateEstimates(struct HEntry *whichHost, RPC2_Unsigned ElapsedTime,
 			  RPC2_Unsigned InBytes, RPC2_Unsigned OutBytes);
-void rpc2_RetryInterval(RPC2_Handle whichConn, RPC2_Unsigned InBytes,
-			RPC2_Unsigned OutBytes, int *retry, int maxretry,
-			struct timeval *tv);
+int rpc2_RetryInterval(struct CEntry *ce, int retry, struct timeval *tv,
+		       RPC2_Unsigned OutBytes, RPC2_Unsigned InBytes);
 
 /* Multicast group manipulation routines */
 void rpc2_InitMgrp(), rpc2_FreeMgrp(), rpc2_RemoveFromMgrp(), rpc2_DeleteMgrp();
@@ -681,4 +690,38 @@ do {\
 
 #define TSDELTA(_ts1_, _ts2_) ((int)(_ts1_) - (int)(_ts2_))
 
+#ifdef USE_LUA
+/* lua.c - include Lua interpreter for roundtrip time estimators */
+void LUA_init(void);
+void LUA_clocktick(void);
+void LUA_drop_hosttable(struct HEntry *he);
+
+void LUA_rtt_update(struct HEntry *he, uint32_t rtt, uint32_t tx, uint32_t rx);
+int LUA_rtt_getrto(struct HEntry *he, uint32_t tx, uint32_t rx);
+int LUA_rtt_getbandwidth(struct HEntry *he, uint32_t *bw_tx, uint32_t *bw_rx);
+/* not sure if we want to allow the estimator to control number of retries
+ * and/or overall timeout, so I might not actually use this function... */
+int LUA_rtt_retryinterval(struct HEntry *he, uint32_t n, uint32_t tx, uint32_t rx);
+int LUA_fail_delay(struct RPC2_addrinfo *addr, RPC2_PacketBuffer *pb, int out,
+		   struct timeval *tv);
+#else
+/* do not include Lua interpreter, define empty stubs */
+#define LUA_init()
+#define LUA_clocktick()
+#define LUA_drop_hosttable(a)
+#define LUA_rtt_update(a,b,c,d)
+#define LUA_rtt_getrto(a,b,c)		0
+#define LUA_rtt_getbandwidth(a,b,c)	0
+#define LUA_rtt_retryinterval(a,b,c,d)	0
+#define LUA_fail_delay(a,b,c,d)		0
 #endif
+
+int rpc2_DelayedSend(int s, struct RPC2_addrinfo *addr, RPC2_PacketBuffer *pb,
+		     struct timeval *delay);
+int rpc2_DelayedRecv(RPC2_PacketBuffer *pb, struct timeval *delay);
+
+void rpc2_SendDelayedPacket(struct SL_Entry *sl);
+RPC2_PacketBuffer *rpc2_RecvDelayedPacket(struct SL_Entry *sl);
+
+#endif /* _RPC2_PRIVATE_H_ */
+
