@@ -56,8 +56,6 @@ Pittsburgh, PA.
 #include "trace.h"
 #include "cbuf.h"
 
-
-
 /*
 Contains the hard core of the major RPC runtime routines.
 
@@ -117,10 +115,12 @@ static int InvokeSE();
 static void SendOKInit2();
 static int ServerHandShake(struct CEntry *ce, int32_t xrand,
 			   RPC2_EncryptionKey SharedSecret,
-			   size_t keysize, int new_binding);
+			   uint32_t rpc2sec_version, size_t keysize,
+			   int new_binding);
 static RPC2_PacketBuffer *Send2Get3(struct CEntry *ce, RPC2_EncryptionKey key,
 				    int32_t xrand, int32_t *yrand,
-				    size_t keysize, int new_binding);
+				    uint32_t rpc2sec_version, size_t keysize,
+				    int new_binding);
 static long Test3(RPC2_PacketBuffer *pb, struct CEntry *ce, int32_t yrand,
 		  RPC2_EncryptionKey ekey, int new_binding);
 static void Send4AndSave(struct CEntry *ce, int32_t xrand,
@@ -129,7 +129,7 @@ static void RejectBind(struct CEntry *ce, size_t bodysize, RPC2_Integer opcode);
 static RPC2_PacketBuffer *HeldReq(RPC2_RequestFilter *filter, struct CEntry **ce);
 static int GetFilter(RPC2_RequestFilter *inf, RPC2_RequestFilter *outf);
 static long GetNewRequest(IN RPC2_RequestFilter *filter, IN struct timeval *timeout, OUT struct RPC2_PacketBuffer **pb, OUT struct CEntry **ce);
-static long MakeFake(INOUT RPC2_PacketBuffer *pb, IN struct CEntry *ce, RPC2_Integer *AuthenticationType, OUT RPC2_Integer *xrand, OUT RPC2_CountedBS *cident, size_t *keysize);
+static long MakeFake(INOUT RPC2_PacketBuffer *pb, IN struct CEntry *ce, RPC2_Integer *AuthenticationType, OUT RPC2_Integer *xrand, OUT RPC2_CountedBS *cident, OUT uint32_t *rpc2sec_version, OUT size_t *keysize);
 
 FILE *rpc2_logfile;
 FILE *rpc2_tracefile;
@@ -164,7 +164,7 @@ long RPC2_SendResponse(IN RPC2_Handle ConnHandle, IN RPC2_PacketBuffer *Reply)
     long rc;
 
     rpc2_Enter();
-    say(0, RPC2_DebugLevel, "RPC2_SendResponse()\n");
+    say(1, RPC2_DebugLevel, "RPC2_SendResponse()\n");
     assert(!Reply || Reply->Prefix.MagicNumber == OBJ_PACKETBUFFER);
 
     /* Perform sanity checks */
@@ -230,7 +230,8 @@ long RPC2_SendResponse(IN RPC2_Handle ConnHandle, IN RPC2_PacketBuffer *Reply)
 }
 
 /* Helpers to set up keys and pack/unpack init2/init3 packets */
-static int setup_init1_key(int (*init)(struct security_association *sa,
+static int setup_init1_key(int (*init)(uint32_t secure_version,
+				       struct security_association *sa,
 				       const struct secure_auth *auth,
 				       const struct secure_encr *encr,
 				       const uint8_t *key, size_t len),
@@ -256,7 +257,7 @@ static int setup_init1_key(int (*init)(struct security_association *sa,
 		      key, sizeof(key));
     if (rc) return -1;
 
-    rc = init(sa, auth, encr, key, sizeof(key));
+    rc = init(0, sa, auth, encr, key, sizeof(key));
     memset(key, 0, sizeof(key));
     return rc;
 }
@@ -264,23 +265,27 @@ static int setup_init1_key(int (*init)(struct security_association *sa,
 static int pack_initX_body(struct security_association *sa,
 			   const struct secure_auth *auth,
 			   const struct secure_encr *encr,
+			   uint32_t rpc2sec_version,
 			   void *packet_body, size_t len)
 {
 	uint32_t *body = (uint32_t *)packet_body;
 
-	body[0] = htonl(0); /* version */
+	if (!auth || !encr) return -1;
+
+	body[0] = htonl(rpc2sec_version); /* version */
 	body[1] = htonl(auth->id);
 	body[2] = htonl(encr->id);
 	secure_random_bytes(&body[3], len);
 
-	return secure_setup_decrypt(sa, auth, encr, (uint8_t *)&body[3], len);
+	return secure_setup_decrypt(rpc2sec_version, sa, auth, encr,
+				    (uint8_t *)&body[3], len);
 }
 
 static int unpack_initX_body(struct CEntry *ce,
 			     struct RPC2_PacketBuffer *pb,
 			     const struct secure_auth **auth,
 			     const struct secure_encr **encr,
-			     size_t *keylen)
+			     uint32_t *rpc2sec_version, size_t *keylen)
 {
     const struct secure_auth *a;
     const struct secure_encr *e;
@@ -300,8 +305,10 @@ static int unpack_initX_body(struct CEntry *ce,
 
     /* lookup authentication and encryption functions */
     version = ntohl(body[0]);
-    if (version != 0)
-	return RPC2_NOTAUTHENTICATED;
+
+    /* Our peer should have switched to the lowest common version. If it hasn't
+     * that is likely a good indication that something is very wrong. */
+    if (version > SECURE_VERSION) return RPC2_NOTAUTHENTICATED;
 
     a = secure_get_auth_byid(ntohl(body[1]));
     e = secure_get_encr_byid(ntohl(body[2]));
@@ -318,12 +325,13 @@ static int unpack_initX_body(struct CEntry *ce,
 	return RPC2_NOTAUTHENTICATED;
 
     /* initialize keys */
-    rc = secure_setup_encrypt(&ce->sa, a, e, (uint8_t *)&body[3], len);
+    rc = secure_setup_encrypt(version, &ce->sa, a, e, (uint8_t *)&body[3], len);
     if (rc) return RPC2_NOTAUTHENTICATED;
 
-    if (auth)   *auth = a;
-    if (encr)   *encr = e;
-    if (keylen) *keylen = len;
+    if (rpc2sec_version) *rpc2sec_version = version;
+    if (auth)	*auth = a;
+    if (encr)	*encr = e;
+    if (keylen)	*keylen = len;
     return RPC2_SUCCESS;
 }
 
@@ -343,11 +351,12 @@ long RPC2_GetRequest(IN RPC2_RequestFilter *Filter,
 	RPC2_CountedBS cident, *clientIdent;
 	RPC2_Integer XRandom;
 	RPC2_EncryptionKey SharedSecret;
+	uint32_t rpc2sec_version = 0;
 	size_t keysize = 0;
 	long rc;
 
 	rpc2_Enter();
-	say(0, RPC2_DebugLevel, "RPC2_GetRequest()\n");
+	say(1, RPC2_DebugLevel, "RPC2_GetRequest()\n");
 
 	TR_GETREQUEST();
 
@@ -414,7 +423,8 @@ long RPC2_GetRequest(IN RPC2_RequestFilter *Filter,
 
     /* Extract relevant fields from Init1 packet and then make it a fake
      * NEWCONNECTION packet */
-    rc = MakeFake(pb, ce, &XRandom, &AuthenticationType, &cident, &keysize);
+    rc = MakeFake(pb, ce, &XRandom, &AuthenticationType, &cident,
+		  &rpc2sec_version, &keysize);
     if (rc < RPC2_WLIMIT) {DROPIT();}
 
     memset(SharedSecret, 0, sizeof(RPC2_EncryptionKey));
@@ -439,7 +449,8 @@ long RPC2_GetRequest(IN RPC2_RequestFilter *Filter,
     }
 
     /* new bind sequence */
-    if (pb->Header.Flags & RPC2SEC_CAPABLE) {
+    if (pb->Header.Flags & RPC2SEC_CAPABLE)
+    {
 	/* setup xmit key so we can reply with encrypted key material */
 	rc = setup_init1_key(secure_setup_encrypt, &ce->sa, XRandom,
 			     ce->PeerUnique, SharedSecret);
@@ -449,12 +460,17 @@ long RPC2_GetRequest(IN RPC2_RequestFilter *Filter,
 	    DROPIT();
 	}
 
+	/* use the lowest common denominator */
+	if (rpc2sec_version > SECURE_VERSION)
+	    rpc2sec_version = SECURE_VERSION;
+
 	/* why reimplement something that is already working... */
 	/* we just need some minor tweaks for the existing INIT2/INIT3/INIT4
 	 * sequence, but this is should not be a problem, the packets in the
 	 * new handshake are easily recognized by having a non-NULL
 	 * pb->Prefix.sa field */
-	rc = ServerHandShake(ce, XRandom, SharedSecret, keysize, 1);
+	rc = ServerHandShake(ce, XRandom, SharedSecret, rpc2sec_version,
+			     keysize, 1);
 	if (rc != RPC2_SUCCESS)
 	    DROPIT();
     }
@@ -473,7 +489,7 @@ long RPC2_GetRequest(IN RPC2_RequestFilter *Filter,
 	/* It is probably safer to ignore the SessionKey returned by GetKeys. */
 	secure_random_bytes(ce->SessionKey, sizeof(RPC2_EncryptionKey));
 
-	rc = ServerHandShake(ce, XRandom, SharedSecret, 0, 0);
+	rc = ServerHandShake(ce, XRandom, SharedSecret, 0, 0, 0);
 	if (rc != RPC2_SUCCESS)
 	    DROPIT();
     }
@@ -519,7 +535,7 @@ long RPC2_MakeRPC(RPC2_Handle ConnHandle, RPC2_PacketBuffer *Request,
     long rc, secode = RPC2_SUCCESS, finalrc, opcode;
 
     rpc2_Enter();
-    say(0, RPC2_DebugLevel, "RPC2_MakeRPC()\n");
+    say(1, RPC2_DebugLevel, "RPC2_MakeRPC()\n");
 
     TR_MAKERPC();
 
@@ -541,9 +557,9 @@ long RPC2_MakeRPC(RPC2_Handle ConnHandle, RPC2_PacketBuffer *Request,
 	if (SDesc && ce->sebroken) rpc2_Quit(RPC2_SEFAIL2);
 
 	if (!EnqueueRequest) rpc2_Quit(RPC2_CONNBUSY);
-	say(0, RPC2_DebugLevel, "Enqueuing on connection %#x\n",ConnHandle);
+	say(1, RPC2_DebugLevel, "Enqueuing on connection %#x\n",ConnHandle);
 	LWP_WaitProcess((char *)ce);
-	say(0, RPC2_DebugLevel, "Dequeueing on connection %#x\n", ConnHandle);
+	say(1, RPC2_DebugLevel, "Dequeueing on connection %#x\n", ConnHandle);
 	}
     /* XXXXXX race condition with preemptive threads */
     SetState(ce, C_AWAITREPLY);
@@ -557,8 +573,7 @@ long RPC2_MakeRPC(RPC2_Handle ConnHandle, RPC2_PacketBuffer *Request,
     rpc2_InitPacket(preq, ce, preq->Header.BodyLength);
     preq->Header.SeqNumber = ce->NextSeqNumber;
     preq->Header.Opcode = opcode;
-    preq->Header.BindTime = ce->RTT >> RPC2_RTT_SHIFT;  /* bind time on 1st rpc */
-    if (ce->RTT && preq->Header.BindTime == 0) preq->Header.BindTime = 1;  /* ugh */
+    preq->Header.BindTime = 0;
 
     /* Notify side effect routine, if any */
     if (SDesc && HAVE_SE_FUNC(SE_MakeRPC1))
@@ -604,7 +619,7 @@ long RPC2_MakeRPC(RPC2_Handle ConnHandle, RPC2_PacketBuffer *Request,
 	case ARRIVED:
 		say(9, RPC2_DebugLevel,
 			"Request reliably sent on %#x\n", ConnHandle);
-		*Reply = preply = sl->Packet;
+		*Reply = preply = (RPC2_PacketBuffer *)sl->data;
 		rpc2_FreeSle(&sl);
 		/* release packet allocated by SE routine */
 		if (preq != Request) RPC2_FreeBuffer(&preq);
@@ -692,13 +707,14 @@ long RPC2_NewBinding(IN RPC2_HostIdent *Host, IN RPC2_PortIdent *Port,
     int new_binding = 0;
     const struct secure_auth *auth = NULL;
     const struct secure_encr *encr = NULL;
+    uint32_t rpc2sec_version = 0;
     size_t keylen = 0;
 
 #define DROPCONN()\
 	    {rpc2_SetConnError(ce); (void) RPC2_Unbind(*ConnHandle); *ConnHandle = 0;}
 
     rpc2_Enter();
-    say(0, RPC2_DebugLevel, "In RPC2_NewBinding()\n");
+    say(1, RPC2_DebugLevel, "In RPC2_NewBinding()\n");
 
     TR_BIND();
 
@@ -751,7 +767,7 @@ try_next_addr:
     rc = setup_init1_key(secure_setup_decrypt, &ce->sa, xrandom, ce->PeerUnique,
 			 rpc2key);
     if (rc) {
-	say(0, RPC2_DebugLevel, "Failed to initialize security context\n");
+	say(1, RPC2_DebugLevel, "Failed to initialize security context\n");
 	rpc2_Quit(RPC2_FAIL);
     }
 
@@ -816,16 +832,17 @@ try_next_addr:
     /* Fill in the body */
     ib = (struct Init1Body *)pb->Body;
     memset(ib, 0, sizeof(struct Init1Body));
-    ib->FakeBody.SideEffectType = htonl(Bparms->SideEffectType);
-    ib->FakeBody.SecurityLevel = htonl(Bparms->SecurityLevel);
-    ib->FakeBody.EncryptionType = htonl(Bparms->EncryptionType);
-    ib->FakeBody.AuthenticationType = htonl(Bparms->AuthenticationType);
+    ib->FakeBody_SideEffectType = htonl(Bparms->SideEffectType);
+    ib->FakeBody_SecurityLevel = htonl(Bparms->SecurityLevel);
+    ib->FakeBody_EncryptionType = htonl(Bparms->EncryptionType);
+    ib->FakeBody_AuthenticationType = htonl(Bparms->AuthenticationType);
+    ib->FakeBody_ClientIdent_SeqBody = 0;
     if (Bparms->ClientIdent == NULL)
-	ib->FakeBody.ClientIdent.SeqLen = 0;
+	ib->FakeBody_ClientIdent_SeqLen = 0;
     else
 	{
-	ib->FakeBody.ClientIdent.SeqLen = htonl(Bparms->ClientIdent->SeqLen);
-	ib->FakeBody.ClientIdent.SeqBody = ib->Text;    /* not really meaningful: this is pointer has to be reset on other side */
+	ib->FakeBody_ClientIdent_SeqLen = htonl(Bparms->ClientIdent->SeqLen);
+	/* ib->FakeBody_ClientIdent_SeqBody = ib->Text; // not really meaningful: this is pointer has to be reset on other side */
 	memcpy(ib->Text, Bparms->ClientIdent->SeqBody, Bparms->ClientIdent->SeqLen);
 	}
     assert(sizeof(RPC2_VERSION) < sizeof(ib->Version));
@@ -834,7 +851,7 @@ try_next_addr:
     /* hint for the other side that we support the new encryption layer */
     if (ce->sa.decrypt) {
 	pb->Header.Flags |= RPC2SEC_CAPABLE;
-	ib->RPC2SEC_version = htonl(0);
+	ib->RPC2SEC_version = htonl(SECURE_VERSION);
 	ib->Preferred_Keysize = htonl(RPC2_Preferred_Keysize);
     }
 
@@ -857,14 +874,14 @@ try_next_addr:
     say(9, RPC2_DebugLevel, "Sending INIT1 packet on %#x\n", *ConnHandle);
     /* create call entry */
     sl = rpc2_AllocSle(OTHER, ce);
-    rpc2_SendReliably(ce, sl, pb, (struct timeval *)&ce->Retry_Beta[0]);
+    rpc2_SendReliably(ce, sl, pb, &ce->KeepAlive);
 
     switch(sl->ReturnCode)
 	{
 	case ARRIVED:
 		say(9, RPC2_DebugLevel, "Received INIT2 packet on %#x\n", *ConnHandle);
 		RPC2_FreeBuffer(&pb);	/* release the Init1 Packet */
-		pb = sl->Packet;		/* and get the Init2 Packet */
+		pb = (RPC2_PacketBuffer *)sl->data; /* get the Init2 Packet */
 		rpc2_FreeSle(&sl);
 		break;
 
@@ -902,12 +919,22 @@ try_next_addr:
     /* If the reply was over a secure connection, we interpret the INIT2 body
      * differently (uint32_t auth, uint32_t encr, uint8_t key[]) */
     if (pb->Prefix.sa) {
-	rc = unpack_initX_body(ce, pb, &auth, &encr, &keylen);
+	rc = unpack_initX_body(ce, pb, &auth, &encr, &rpc2sec_version, &keylen);
 	if (rc != RPC2_SUCCESS) {
 	    DROPCONN();
 	    RPC2_FreeBuffer(&pb);
 	    rpc2_Quit(rc);
 	}
+	/* When we know that the server is trying to use an incorrectly
+	 * initialized AES-CCM counter, we can't do much about that. However
+	 * we can at least force it to use AES-CBC encryption for any packets
+	 * it sends back to us. */
+	if (rpc2sec_version == 0) {
+	    auth = secure_get_auth_byid(SECURE_AUTH_AES_XCBC_96);
+	    encr = secure_get_encr_byid(SECURE_ENCR_AES_CBC);
+	    keylen += auth->keysize;
+	}
+
 	new_binding = 1;
     }
     else if (RPC2_secure_only) {
@@ -919,7 +946,7 @@ try_next_addr:
     else {
 	/* The INIT2 packet came over insecure connection, remove decryption
 	 * context and fall back on the old handshake */
-	secure_setup_decrypt(&ce->sa, NULL, NULL, NULL, 0);
+	secure_setup_decrypt(0, &ce->sa, NULL, NULL, NULL, 0);
 	say(1, RPC2_DebugLevel, "Got INIT2, proceeding with old binding\n");
 	new_binding = 0;
 
@@ -950,7 +977,7 @@ try_next_addr:
 
     ce->PeerHandle = pb->Header.LocalHandle;
     ce->sa.peer_spi = pb->Header.LocalHandle;
-    say(9, RPC2_DebugLevel, "PeerHandle for local %#x is %#x\n", 
+    say(9, RPC2_DebugLevel, "PeerHandle for local %#x is %#x\n",
 	*ConnHandle, ce->PeerHandle);
     RPC2_FreeBuffer(&pb);	/* Release INIT2 packet */
 
@@ -977,7 +1004,8 @@ try_next_addr:
 	 * - A rogue server won't care how we encrypt outgoing data, it clearly
 	 *   will be getting any required key material with this INIT3 packet.
 	 */
-	rc = pack_initX_body(&ce->sa, auth, encr, &pb->Body, keylen);
+	rc = pack_initX_body(&ce->sa, auth, encr, rpc2sec_version,
+			     &pb->Body, keylen);
 	if (rc) {
 	    DROPCONN();
 	    RPC2_FreeBuffer(&pb);
@@ -997,14 +1025,14 @@ try_next_addr:
 
     /* create call entry */
     sl = rpc2_AllocSle(OTHER, ce);
-    rpc2_SendReliably(ce, sl, pb, (struct timeval *)&ce->Retry_Beta[0]);
+    rpc2_SendReliably(ce, sl, pb, &ce->KeepAlive);
 
     switch(sl->ReturnCode)
 	{
 	case ARRIVED:
 		say(9, RPC2_DebugLevel, "Received INIT4 packet on %#x\n", *ConnHandle);
 		RPC2_FreeBuffer(&pb);	/* release the Init3 Packet */
-		pb = sl->Packet;	/* and get the Init4 Packet */
+		pb = (RPC2_PacketBuffer *)sl->data; /* get the Init4 Packet */
 		rpc2_FreeSle(&sl);
 		break;
 
@@ -1071,15 +1099,12 @@ try_next_addr:
 BindOver:
     /* Call side effect routine if present */
     if (HAVE_SE_FUNC(SE_Bind2)) {
-	RPC2_Unsigned BindTime;
-
-	BindTime = ce->RTT >> RPC2_RTT_SHIFT;
-	if (BindTime == 0) BindTime = 1;  /* ugh. zero is overloaded. */
-	if ((rc = (*ce->SEProcs->SE_Bind2)(*ConnHandle, BindTime)) != RPC2_SUCCESS)
-	    {
+	rc = (*ce->SEProcs->SE_Bind2)(*ConnHandle, 0);
+	if (rc != RPC2_SUCCESS)
+	{
 	    DROPCONN();
 	    rpc2_Quit(rc);
-	    }
+	}
     }
 
     SetState(ce, C_THINK);
@@ -1091,7 +1116,7 @@ BindOver:
 
 long RPC2_InitSideEffect(IN RPC2_Handle ConnHandle, IN SE_Descriptor *SDesc)
 {
-    say(0, RPC2_DebugLevel, "RPC2_InitSideEffect()\n");
+    say(1, RPC2_DebugLevel, "RPC2_InitSideEffect()\n");
 
     TR_INITSE();
 
@@ -1102,7 +1127,7 @@ long RPC2_InitSideEffect(IN RPC2_Handle ConnHandle, IN SE_Descriptor *SDesc)
 long RPC2_CheckSideEffect(IN RPC2_Handle ConnHandle, 
 			  INOUT SE_Descriptor *SDesc, IN long Flags)
 {
-    say(0, RPC2_DebugLevel, "RPC2_CheckSideEffect()\n");
+    say(1, RPC2_DebugLevel, "RPC2_CheckSideEffect()\n");
 
     TR_CHECKSE();
     
@@ -1143,7 +1168,7 @@ long RPC2_Unbind(RPC2_Handle whichConn)
 	struct CEntry *ce;
 	struct MEntry *me;
 	
-	say(0, RPC2_DebugLevel, "RPC2_Unbind(%x)\n", whichConn);
+	say(1, RPC2_DebugLevel, "RPC2_Unbind(%x)\n", whichConn);
 	
 	TR_UNBIND();
 
@@ -1287,7 +1312,7 @@ TryAnother:
 		    
 	case ARRIVED:	/* a request that matches my filter was received */
 		    say(9, RPC2_DebugLevel, "Request wait succeeded\n");
-		    *pb = sl->Packet;	/* save a pointer to the buffer */
+		    *pb = (RPC2_PacketBuffer *)sl->data; /* save the buffer */
 		    rpc2_FreeSle(&sl);
 
 		    *ce = rpc2_GetConn((*pb)->Header.RemoteHandle);
@@ -1308,9 +1333,14 @@ TryAnother:
 
 static long MakeFake(RPC2_PacketBuffer *pb, struct CEntry *ce,
 		     RPC2_Integer *xrand, RPC2_Integer *authenticationtype,
-		     RPC2_CountedBS *cident, size_t *keysize)
+		     RPC2_CountedBS *cident, uint32_t *rpc2sec_version,
+		     size_t *keysize)
 {
-    /* Synthesize fake packet after extracting encrypted XRandom and clientident */
+    /* Synthesize fake packet after extracting encrypted XRandom and
+     * clientident. It is really pretty ugly if you think about it, we
+     * transform the Init1 packet to look like a valid RPC2_NEWCONNECTION
+     * rpc so that it can be unpacked by the stub generated code and passed
+     * back to the application. */
     long i;
     struct Init1Body *ib1;
     RPC2_NewConnectionBody *ncb;
@@ -1322,7 +1352,7 @@ static long MakeFake(RPC2_PacketBuffer *pb, struct CEntry *ce,
     }
 
     ib1 = (struct Init1Body *)(pb->Body);
-    ncb = &ib1->FakeBody;
+    ncb = (RPC2_NewConnectionBody *)&ib1->FakeBody_SideEffectType;
 
     if (strcmp((char *)ib1->Version, RPC2_VERSION) != 0)
 	{
@@ -1333,6 +1363,7 @@ static long MakeFake(RPC2_PacketBuffer *pb, struct CEntry *ce,
 
     *xrand  = ib1->XRandom;		/* Still encrypted */
     *authenticationtype = ntohl(ncb->AuthenticationType);
+    *rpc2sec_version = ntohl(ib1->RPC2SEC_version);
     *keysize = ntohl(ib1->Preferred_Keysize);
 
     cident->SeqLen = ntohl(ncb->ClientIdent.SeqLen);
@@ -1385,7 +1416,8 @@ static void SendOKInit2(IN struct CEntry *ce)
 
 static int ServerHandShake(struct CEntry *ce, int32_t xrand,
 			   RPC2_EncryptionKey SharedSecret,
-			   size_t keysize, int new_binding)
+			   uint32_t rpc2sec_version, size_t keysize,
+			   int new_binding)
 {
     RPC2_PacketBuffer *pb;
     int32_t saveYRandom;
@@ -1398,7 +1430,8 @@ static int ServerHandShake(struct CEntry *ce, int32_t xrand,
     }
 
     /* Send Init2 packet and await Init3 */
-    pb = Send2Get3(ce, SharedSecret, xrand, &saveYRandom, keysize, new_binding);
+    pb = Send2Get3(ce, SharedSecret, xrand, &saveYRandom, rpc2sec_version,
+		   keysize, new_binding);
     if (!pb) return(RPC2_NOTAUTHENTICATED);
 
     /* Validate Init3 */
@@ -1434,7 +1467,8 @@ static void RejectBind(struct CEntry *ce, size_t bodysize, RPC2_Integer opcode)
 
 static RPC2_PacketBuffer *Send2Get3(struct CEntry *ce, RPC2_EncryptionKey key,
 				    int32_t xrand, int32_t *yrand,
-				    size_t keysize, int new_binding)
+				    uint32_t rpc2sec_version, size_t keysize,
+				    int new_binding)
 {
     RPC2_PacketBuffer *pb2, *pb3 = NULL;
     struct Init2Body *ib2;
@@ -1445,8 +1479,13 @@ static RPC2_PacketBuffer *Send2Get3(struct CEntry *ce, RPC2_EncryptionKey key,
     size_t bodylen = sizeof(struct Init2Body);
 
     if (new_binding) {
-	auth = secure_get_auth_byid(SECURE_AUTH_NONE);
-	encr = secure_get_encr_byid(SECURE_ENCR_AES_CCM_8);
+	if (rpc2sec_version == 0) { /* version 0, broken AES-CCM counter */
+	    auth = secure_get_auth_byid(SECURE_AUTH_AES_XCBC_96);
+	    encr = secure_get_encr_byid(SECURE_ENCR_AES_CBC);
+	} else {
+	    auth = secure_get_auth_byid(SECURE_AUTH_NONE);
+	    encr = secure_get_encr_byid(SECURE_ENCR_AES_CCM_8);
+	}
 	if (!auth || !encr) return NULL;
 
 	/* find the longest key that both the client and the server agree on */
@@ -1474,7 +1513,8 @@ static RPC2_PacketBuffer *Send2Get3(struct CEntry *ce, RPC2_EncryptionKey key,
 
     /* and pack the body */
     if (new_binding) {
-	if (pack_initX_body(&ce->sa, auth, encr, &pb2->Body, keysize))
+	if (pack_initX_body(&ce->sa, auth, encr, rpc2sec_version,
+			    &pb2->Body, keysize))
 	{
 	    RPC2_FreeBuffer(&pb2);
 	    return NULL;
@@ -1496,12 +1536,12 @@ static RPC2_PacketBuffer *Send2Get3(struct CEntry *ce, RPC2_EncryptionKey key,
     /* Send Init2 packet and await Init3 packet */
     SetState(ce, S_AWAITINIT3);
     sl = rpc2_AllocSle(OTHER, ce);
-    rpc2_SendReliably(ce, sl, pb2, (struct timeval *)&ce->Retry_Beta[0]);
+    rpc2_SendReliably(ce, sl, pb2, &ce->KeepAlive);
 
     switch(sl->ReturnCode)
 	{
 	case ARRIVED:
-		pb3 = sl->Packet;	/* get the Init3 Packet */
+		pb3 = (RPC2_PacketBuffer *)sl->data; /* get the Init3 Packet */
 		break;
 
 	case NAKED:
@@ -1526,7 +1566,7 @@ static long Test3(RPC2_PacketBuffer *pb, struct CEntry *ce, int32_t yrand,
     struct Init3Body *ib3;
 
     if (new_binding)
-	return unpack_initX_body(ce, pb, NULL, NULL, NULL);
+	return unpack_initX_body(ce, pb, NULL, NULL, NULL, NULL);
 
     if (pb->Prefix.LengthOfPacket < (ssize_t)(sizeof(struct RPC2_PacketHeader) + sizeof(struct Init3Body)))
     {
