@@ -72,7 +72,8 @@ Pittsburgh, PA.
 /*----------------------- Local procedure specs  ----------------------*/
 static long GetFile();
 static long PutFile();
-static RPC2_PacketBuffer *AwaitPacket();
+static RPC2_PacketBuffer *AwaitPacket(struct SFTP_Entry *sEntry, int retry,
+				      int outbytes, int inbytes);
 static long MakeBigEnough();
 
 /*---------------------------  Local macros ---------------------------*/
@@ -99,7 +100,7 @@ static long MakeBigEnough();
 
 long SFTP_Init()
 {
-    say(0, SFTP_DebugLevel, "SFTP_Init()\n");
+    say(1, SFTP_DebugLevel, "SFTP_Init()\n");
 
     sftp_InitTrace();
 
@@ -140,7 +141,7 @@ void SFTP_Activate(initPtr)
 	SFTP_PacketSize = initPtr->PacketSize;
 	SFTP_WindowSize= initPtr->WindowSize;
 	SFTP_RetryCount = initPtr->RetryCount;
-	SFTP_RetryInterval = initPtr->RetryInterval;	/* milliseconds */
+	//SFTP_RetryInterval = initPtr->RetryInterval;	/* milliseconds */
 	SFTP_EnforceQuota = initPtr->EnforceQuota;
 	SFTP_SendAhead = initPtr->SendAhead;
 	SFTP_AckPoint = initPtr->AckPoint;
@@ -191,7 +192,7 @@ long SFTP_Bind1(IN ConnHandle, IN ClientIdent)
     {
     struct SFTP_Entry *se;
 
-    say(0, SFTP_DebugLevel, "SFTP_Bind()\n");
+    say(1, SFTP_DebugLevel, "SFTP_Bind()\n");
 
     se = sftp_AllocSEntry();	/* malloc and initialize SFTP_Entry */
     se->WhoAmI = SFCLIENT;
@@ -205,7 +206,6 @@ long SFTP_Bind1(IN ConnHandle, IN ClientIdent)
 long SFTP_Bind2(IN RPC2_Handle ConnHandle, IN RPC2_Unsigned BindTime)
 {
     struct SFTP_Entry *se;
-    int retry;
 
     assert(RPC2_GetSEPointer(ConnHandle, &se) == RPC2_SUCCESS);
     RPC2_GetPeerInfo(ConnHandle, &se->PInfo);
@@ -218,19 +218,6 @@ long SFTP_Bind2(IN RPC2_Handle ConnHandle, IN RPC2_Unsigned BindTime)
     se->HostInfo = rpc2_GetHost(se->PInfo.RemoteHost.Value.AddrInfo);
     assert(se->HostInfo);
 
-    if (BindTime)
-    {
-        /* XXX Do some estimate of the amount of transferred data --JH */
-	RPC2_UpdateEstimates(se->HostInfo, BindTime,
-			     sizeof(struct RPC2_PacketHeader),
-			     sizeof(struct RPC2_PacketHeader));
-
-	retry = 1;
-        rpc2_RetryInterval(ConnHandle, sizeof(struct RPC2_PacketHeader),
-			   sizeof(struct RPC2_PacketHeader), &retry,
-			   se->RetryCount, &se->RInterval);
-    }
-    
     return(RPC2_SUCCESS);
 }
 
@@ -253,7 +240,7 @@ long SFTP_NewConn(IN ConnHandle, IN ClientIdent)
     {
     struct SFTP_Entry *se;
 
-    say(0, SFTP_DebugLevel, "SFTP_NewConn()\n");
+    say(1, SFTP_DebugLevel, "SFTP_NewConn()\n");
 
     se = sftp_AllocSEntry();	/* malloc and initialize */
     se->WhoAmI = SFSERVER;
@@ -284,7 +271,7 @@ long SFTP_MakeRPC1(IN ConnHandle, INOUT SDesc, INOUT RequestPtr)
     int rc;
     off_t len;
 
-    say(0, SFTP_DebugLevel, "SFTP_MakeRPC1 ()\n");
+    say(1, SFTP_DebugLevel, "SFTP_MakeRPC1 ()\n");
 
     SDesc->LocalStatus = SE_SUCCESS;	/* non-execution == success */
     SDesc->RemoteStatus = SE_SUCCESS;	/* non-execution == success */
@@ -350,7 +337,7 @@ long SFTP_MakeRPC2(IN ConnHandle, INOUT SDesc, INOUT Reply)
     int i;
     off_t nbytes;
 
-    say(0, SFTP_DebugLevel, "SFTP_MakeRPC2()\n");
+    say(1, SFTP_DebugLevel, "SFTP_MakeRPC2()\n");
     
     assert(RPC2_GetSEPointer(ConnHandle, &se) == RPC2_SUCCESS);
 
@@ -405,26 +392,13 @@ long SFTP_GetRequest(RPC2_Handle ConnHandle, RPC2_PacketBuffer *Request)
 {
     struct SFTP_Entry *se;
     off_t len;
-    int retry;
 
-    say(0, SFTP_DebugLevel, "SFTP_GetRequest()\n");
+    say(1, SFTP_DebugLevel, "SFTP_GetRequest()\n");
 
     assert (RPC2_GetSEPointer(ConnHandle, &se) == RPC2_SUCCESS &&  se != NULL);
     if (se->WhoAmI != SFSERVER) FAIL(se, RPC2_SEFAIL2);
     se->ThisRPCCall = Request->Header.SeqNumber;   /* acquire client's RPC call number */
 
-    if (Request->Header.BindTime)
-    {
-        /* XXX Do some estimate of the amount of transferred data --JH */
-	RPC2_UpdateEstimates(se->HostInfo, Request->Header.BindTime,
-			     sizeof(struct RPC2_PacketHeader),
-			     sizeof(struct RPC2_PacketHeader));
-	retry = 1;
-        rpc2_RetryInterval(ConnHandle, sizeof(struct RPC2_PacketHeader),
-			   sizeof(struct RPC2_PacketHeader), &retry,
-			   se->RetryCount, &se->RInterval);
-    }
-
     se->PiggySDesc = NULL; /* default is no piggybacked file */
     if ((Request->Header.SEFlags & SFTP_PIGGY))
 	{
@@ -453,7 +427,7 @@ long SFTP_InitSE(RPC2_Handle ConnHandle, SE_Descriptor *SDesc)
     struct SFTP_Entry *se;
     int rc;
 
-    say(0, SFTP_DebugLevel, "SFTP_InitSE ()\n");
+    say(1, SFTP_DebugLevel, "SFTP_InitSE ()\n");
 	
     SDesc->LocalStatus = SE_NOTSTARTED;
     SDesc->RemoteStatus = SE_NOTSTARTED;
@@ -481,7 +455,7 @@ long SFTP_CheckSE(RPC2_Handle ConnHandle, SE_Descriptor *SDesc, long Flags)
     struct FileInfoByAddr *p;
 	
 
-    say(0, SFTP_DebugLevel, "SFTP_CheckSE()\n");
+    say(1, SFTP_DebugLevel, "SFTP_CheckSE()\n");
 
     if (Flags == 0) return(RPC2_SUCCESS);
     assert (RPC2_GetSEPointer(ConnHandle, &se) == RPC2_SUCCESS &&  se != NULL);
@@ -498,19 +472,19 @@ long SFTP_CheckSE(RPC2_Handle ConnHandle, SE_Descriptor *SDesc, long Flags)
 	switch(sftpd->Tag)
 	    {
 	    case FILEBYNAME:
-		say(0, SFTP_DebugLevel, "%s: ", sftpd->FileInfo.ByName.LocalFileName);
+		say(1, SFTP_DebugLevel, "%s: ", sftpd->FileInfo.ByName.LocalFileName);
 	    	break;
 
 	    case FILEBYINODE:
-		say(0, SFTP_DebugLevel, "%ld.%ld: ", sftpd->FileInfo.ByInode.Device, sftpd->FileInfo.ByInode.Inode);
+		say(1, SFTP_DebugLevel, "%ld.%ld: ", sftpd->FileInfo.ByInode.Device, sftpd->FileInfo.ByInode.Inode);
 	    	break;
 
 	    case FILEBYFD:
-		say(0, SFTP_DebugLevel, "%ld: ", sftpd->FileInfo.ByFD.fd);
+		say(1, SFTP_DebugLevel, "%ld: ", sftpd->FileInfo.ByFD.fd);
 	    	break;
 
 	    case FILEINVM:
-		say(0, SFTP_DebugLevel, "%p[%u, %u]: ", sftpd->FileInfo.ByAddr.vmfile.SeqBody, sftpd->FileInfo.ByAddr.vmfile.MaxSeqLen,  sftpd->FileInfo.ByAddr.vmfile.SeqLen);
+		say(1, SFTP_DebugLevel, "%p[%u, %u]: ", sftpd->FileInfo.ByAddr.vmfile.SeqBody, sftpd->FileInfo.ByAddr.vmfile.MaxSeqLen,  sftpd->FileInfo.ByAddr.vmfile.SeqLen);
 	    	break;
 	    }
 
@@ -582,7 +556,7 @@ long SFTP_SendResponse(IN ConnHandle, IN Reply)
     struct SFTP_Entry *se;
     long rc;
 
-    say(0, SFTP_DebugLevel, "SFTP_SendResponse()\n");
+    say(1, SFTP_DebugLevel, "SFTP_SendResponse()\n");
 
     assert (RPC2_GetSEPointer(ConnHandle, &se) == RPC2_SUCCESS &&  se != NULL);
     
@@ -645,7 +619,7 @@ long SFTP_GetTime(IN RPC2_Handle ConnHandle, INOUT struct timeval *Time)
     struct SFTP_Entry *se;
     long rc;
 
-    say(0, SFTP_DebugLevel, "SFTP_GetTime()\n");
+    say(1, SFTP_DebugLevel, "SFTP_GetTime()\n");
 
     se = NULL;
     /* 
@@ -669,7 +643,7 @@ long SFTP_GetHostInfo(IN ConnHandle, INOUT HPtr)
     struct SFTP_Entry *se;
     long rc;
 
-    say(0, SFTP_DebugLevel, "SFTP_GetHostInfo()\n");
+    say(1, SFTP_DebugLevel, "SFTP_GetHostInfo()\n");
 
     se = NULL;
     if ((rc = RPC2_GetSEPointer(ConnHandle, &se)) != RPC2_SUCCESS)
@@ -713,13 +687,9 @@ static long GetFile(struct SFTP_Entry *sEntry)
      * of the next data packet */
     packetsize = sEntry->PacketSize + sizeof(struct RPC2_PacketHeader);
     while (sEntry->XferState == XferInProgress) {
-	for (i = 1; i <= sEntry->RetryCount; i++) {
-	    /* get a new retry interval estimate */
-	    rpc2_RetryInterval(sEntry->LocalHandle, packetsize,
-			       sizeof(struct RPC2_PacketHeader), &i,
-			       sEntry->RetryCount, &sEntry->RInterval);
-
-	    pb = (RPC2_PacketBuffer *)AwaitPacket(&sEntry->RInterval, sEntry);
+	for (i = 0; i < sEntry->RetryCount; i++) {
+	    pb = AwaitPacket(sEntry, i, packetsize,
+			     sizeof(struct RPC2_PacketHeader));
 
 	    /* Make sure nothing bad happened while we were waiting */
 	    if (sEntry->WhoAmI == ERROR) {
@@ -831,12 +801,8 @@ static long PutFile(struct SFTP_Entry *sEntry)
 	     sEntry->AckPoint);
 
     while (sEntry->XferState == XferInProgress) {
-	for (i = 1; i <= sEntry->RetryCount; i++) {
-	    /* get a new retry interval estimate */
-	    rpc2_RetryInterval(sEntry->LocalHandle, sizeof(struct RPC2_PacketHeader),
-			       bytes, &i, sEntry->RetryCount, &sEntry->RInterval);
-
-	    pb = (RPC2_PacketBuffer *)AwaitPacket(&sEntry->RInterval, sEntry);
+	for (i = 0; i < sEntry->RetryCount; i++) {
+	    pb = AwaitPacket(sEntry, i, sizeof(struct RPC2_PacketHeader),bytes);
 
 	    /* Make sure nothing bad happened while we were waiting */
 	    if (sEntry->WhoAmI == ERROR) {
@@ -888,13 +854,15 @@ GotAck:
 }
 
 
-static RPC2_PacketBuffer *AwaitPacket(struct timeval *tOut,
-				      struct SFTP_Entry *sEntry)
-    /* Awaits for a packet on sEntry or  timeout tOut
+static RPC2_PacketBuffer *AwaitPacket(struct SFTP_Entry *sEntry, int retry,
+				      int outbytes, int inbytes)
+    /* Awaits for a packet on sEntry
 	Returns pointer to arrived packet or NULL
     */
 {
     struct SL_Entry *sl;
+    struct CEntry *ce;
+    int rc;
 
     if (LWP_GetRock(SMARTFTP, (void *)&sl) != LWP_SUCCESS)
     {
@@ -902,21 +870,24 @@ static RPC2_PacketBuffer *AwaitPacket(struct timeval *tOut,
 	assert(LWP_NewRock(SMARTFTP, (char *)sl) == LWP_SUCCESS);
     }
 
+    ce = rpc2_GetConn(sEntry->LocalHandle);
+    rc = rpc2_RetryInterval(ce, retry, &sl->RInterval, outbytes, inbytes);
+    if (rc) { sl->ReturnCode = 0; return NULL; }
+
     sEntry->Sleeper = sl;
-    rpc2_ActivateSle(sl, tOut);
+    rpc2_ActivateSle(sl, &sl->RInterval);
     LWP_WaitProcess((char *)sl);
 
     switch(sl->ReturnCode)
 	{
 	case TIMEOUT:	sl->ReturnCode = 0; return(NULL);
-	case ARRIVED:	sl->ReturnCode = 0; return(sl->Packet);
+	case ARRIVED:	sl->ReturnCode = 0; return((RPC2_PacketBuffer *)sl->data);
 	default: assert(FALSE);
 	}
     /*NOTREACHED*/
     assert(0);
     return NULL;
 }
-    
 
 /*---------------------- Piggybacking routines -------------------------*/
 
@@ -991,6 +962,7 @@ off_t sftp_AppendFileToPacket(struct SFTP_Entry *sEntry,
 {
     long rc, maxbytes;
     off_t filelen;
+    struct CEntry *ce;
     static char GlobalJunk[SFTP_MAXBODYSIZE];	/* buffer for read();
 				    avoids huge local on my stack */
     
@@ -1013,6 +985,8 @@ off_t sftp_AppendFileToPacket(struct SFTP_Entry *sEntry,
     if (rc < 0) return(-1);
     assert(!sftp_AddPiggy(whichP, GlobalJunk, filelen, SFTP_MAXPACKETSIZE));
     sEntry->HitEOF = TRUE;
+    ce = rpc2_GetConn(sEntry->LocalHandle);
+    if (ce) ce->reqsize += filelen;
 
     /* cleanup and quit */
     sftp_vfclose(sEntry);
@@ -1113,7 +1087,15 @@ int sftp_ExtractParmsFromPacket(struct SFTP_Entry *sEntry,
 	sEntry->DupThreshold = sp.DupThreshold;
 	}
     sEntry->GotParms = TRUE;
-    say(9, SFTP_DebugLevel, "GotParms: %ld %ld %ld %ld %ld\n", sEntry->WindowSize, sEntry->SendAhead, sEntry->AckPoint, sEntry->PacketSize, sEntry->DupThreshold);
+
+    if (sEntry->WindowSize < SFTP_MINWINDOWSIZE)
+	sEntry->WindowSize = SFTP_MINWINDOWSIZE;
+    if (sEntry->SendAhead < SFTP_MINSENDAHEAD)
+        sEntry->SendAhead = SFTP_MINSENDAHEAD;
+    if (sEntry->PacketSize < SFTP_MINPACKETSIZE)
+	sEntry->PacketSize = SFTP_MINPACKETSIZE;
+
+    say(9, SFTP_DebugLevel, "GotParms: %d %d %d %d %d\n", sEntry->WindowSize, sEntry->SendAhead, sEntry->AckPoint, sEntry->PacketSize, sEntry->DupThreshold);
 
     whichP->Header.BodyLength -= sizeof(struct SFTP_Parms);
 
@@ -1136,8 +1118,6 @@ struct SFTP_Entry *sftp_AllocSEntry(void)
     sfp->SendAhead = SFTP_SendAhead;
     sfp->AckPoint = SFTP_AckPoint;
     sfp->DupThreshold = SFTP_DupThreshold;
-    sfp->RInterval.tv_sec = SFTP_RetryInterval/1000;
-    sfp->RInterval.tv_usec = (SFTP_RetryInterval*1000) % 1000000;
     sfp->Retransmitting = FALSE;
     sfp->RequestTime = 0;
     CLRTIME(&sfp->LastWord);
