@@ -97,6 +97,8 @@ void sftp_ExaminePacket(RPC2_PacketBuffer *pb)
     /* check if the packet and the connection use the same security context. */
     if (ce && (ce->sa.decrypt || ce->sa.validate) && &ce->sa != pb->Prefix.sa)
     {
+	say(1, SFTP_DebugLevel,
+	    "Incoming sftp packet with different security association\n");
 	SFTP_FreeBuffer(&pb);
 	return;
     }
@@ -126,7 +128,7 @@ void sftp_ExaminePacket(RPC2_PacketBuffer *pb)
     if (pb->Header.Opcode == SFTP_NAK)
     {
 	sftp_Recvd.Naks++;
-	say(0, SFTP_DebugLevel, "SFTP_NAK received\n");
+	say(1, SFTP_DebugLevel, "SFTP_NAK received\n");
 
 	/* sfp->WhoAmI is set to ERROR by sftp_SetError */
 	iamserver = (sfp->WhoAmI == SFSERVER);
@@ -145,7 +147,7 @@ void sftp_ExaminePacket(RPC2_PacketBuffer *pb)
      * I disabled this for now */
     if (0 && !RPC2_cmpaddrinfo(sfp->HostInfo->Addr, pb->Prefix.PeerAddr))
     {
-	say(0, SFTP_DebugLevel, "Received SFTP packet from unexpected host\n");
+	say(1, SFTP_DebugLevel, "Received SFTP packet from unexpected host\n");
 	SFSendNAK(pb); /* NAK this packet */
 	BOGUS(pb);
 	return;
@@ -154,7 +156,7 @@ void sftp_ExaminePacket(RPC2_PacketBuffer *pb)
     /* SANITY CHECK: make sure this pertains to the current RPC call. */
     if (pb->Header.ThisRPCCall != sfp->ThisRPCCall)
     {
-	say(0, SFTP_DebugLevel, "Old SFTP packet RPC %d, expecting RPC %ld\n", pb->Header.ThisRPCCall, sfp->ThisRPCCall);
+	say(1, SFTP_DebugLevel, "Old SFTP packet RPC %d, expecting RPC %d\n", pb->Header.ThisRPCCall, sfp->ThisRPCCall);
 	SFTP_FreeBuffer(&pb);
 	return;
     }
@@ -206,7 +208,7 @@ static void ServerPacket(RPC2_PacketBuffer *whichPacket,
     }
     sEntry->Sleeper = NULL;	/* no longer anyone waiting for a packet */
     rpc2_DeactivateSle(sl, ARRIVED);
-    sl->Packet = whichPacket;
+    sl->data = whichPacket;
     LWP_SignalProcess((char *)sl);
 }
 
@@ -214,8 +216,6 @@ static void ServerPacket(RPC2_PacketBuffer *whichPacket,
 static void ClientPacket(RPC2_PacketBuffer *whichPacket,
 			 struct SFTP_Entry *sEntry)
 {
-    unsigned long bytes;
-    int retry;
     /* Deal with this packet on Listener's thread of control */
 
     switch ((int) whichPacket->Header.Opcode)
@@ -227,18 +227,6 @@ static void ClientPacket(RPC2_PacketBuffer *whichPacket,
     case SFTP_ACK:
 	/* Makes sense only if we are on source side */
 	if (IsSource(sEntry)) {
-	    /* we need to get some indication of a retry interval, so that
-	     * AckArrived->SendStrategy->CheckWorried() can actually do
-	     * the right thing */
-
-	    /* estimated size of an sftp data transfer */
-	    bytes = ((sEntry->PacketSize + sizeof(struct RPC2_PacketHeader)) *
-		     sEntry->SendAhead);
-
-	    retry = 1;
-	    rpc2_RetryInterval(sEntry->LocalHandle, sizeof(struct RPC2_PacketHeader),
-			       bytes, &retry, sEntry->RetryCount, &sEntry->RInterval);
-
 	    if (sftp_AckArrived(whichPacket, sEntry) < 0) {
 		SFSendNAK(whichPacket); /* NAK this packet */
 		sftp_SetError(sEntry, ERROR);
@@ -295,7 +283,7 @@ static void SFSendNAK(RPC2_PacketBuffer *pb)
     if (remoteHandle == -1) return;
 
     sftp_Sent.Naks++;
-    say(0, SFTP_DebugLevel, "SFSendNAK\n");
+    say(1, SFTP_DebugLevel, "SFSendNAK\n");
     SFTP_AllocBuffer(0, &nakpb);
     nakpb->Prefix.LengthOfPacket = sizeof(struct RPC2_PacketHeader);
     nakpb->Header.ProtoVersion = SFTPVERSION;
