@@ -67,7 +67,6 @@ Pittsburgh, PA.
 long SFTP_PacketSize;
 long SFTP_WindowSize;
 long SFTP_RetryCount;
-long SFTP_RetryInterval;	/* milliseconds */
 long SFTP_EnforceQuota;
 long SFTP_SendAhead;
 long SFTP_AckPoint;
@@ -103,7 +102,7 @@ void B_CopyFromPacket(RPC2_PacketBuffer *whichPacket, unsigned int *bMask);
 
 #ifdef RPC2DEBUG
 void PrintDb(struct SFTP_Entry *se, RPC2_PacketBuffer *pb);
-#define BOGOSITY(se, pb)  (printf("SFTP bogosity:  file %s, line %d\n", __FILE__, __LINE__), PrintDb(se, pb))
+#define BOGOSITY(se, pb)  (fprintf(rpc2_tracefile, "SFTP bogosity:  file %s, line %d\n", __FILE__, __LINE__), PrintDb(se, pb))
 #else 
 #define BOGOSITY(se, pb)
 #endif
@@ -245,7 +244,7 @@ void sftp_UpdateBW(RPC2_PacketBuffer *pb, unsigned long inbytes,
     entry.Value.Measured.Bytes = inbytes + outbytes;
     entry.Value.Measured.ElapsedTime = obs;
     (void) rpc2_AppendHostLog(sEntry->HostInfo, &entry, SE_MEASUREMENT);
-    say(0/*4*/, SFTP_DebugLevel, "sftp_UpdateBW: conn %#x, %ld inbytes, %ld outbytes, %ld ms\n", 
+    say(1/*4*/, SFTP_DebugLevel, "sftp_UpdateBW: conn %#x, %ld inbytes, %ld outbytes, %ld ms\n", 
 	sEntry->LocalHandle, inbytes, outbytes, obs);
 }
 
@@ -261,7 +260,6 @@ int sftp_DataArrived(RPC2_PacketBuffer *pBuff, struct SFTP_Entry *sEntry)
 {
     long moffset;	/* bit position of TheseBits corresponding to pBuff */
     long i, j;
-    int retry;
     
     if (sEntry->SentParms == FALSE && sEntry->WhoAmI == SFSERVER)
 	sEntry->SentParms = TRUE;    /* this data packet is evidence that my Start got to the client */
@@ -289,9 +287,9 @@ int sftp_DataArrived(RPC2_PacketBuffer *pBuff, struct SFTP_Entry *sEntry)
 	return(0); 
     }
 
-    moffset = pBuff->Header.SeqNumber-sEntry->RecvLastContig;
+    moffset = (long)pBuff->Header.SeqNumber - (long)sEntry->RecvLastContig;
 
-    if (moffset > sEntry->WindowSize) {
+    if (moffset > (long)sEntry->WindowSize) {
 	BOGOSITY(sEntry, pBuff);
 	return(-1);
     }
@@ -366,12 +364,6 @@ int sftp_DataArrived(RPC2_PacketBuffer *pBuff, struct SFTP_Entry *sEntry)
 		 * was actually sent if we sent duplicate packets! -JH */
 		sftp_UpdateBW(pBuff, dataThisRound,
 			      sizeof(struct RPC2_PacketHeader), sEntry);
-
-	    /* recalculate the retry timeout */
-	    retry = 1;
-	    rpc2_RetryInterval(sEntry->LocalHandle, dataThisRound,
-			       sizeof(struct RPC2_PacketHeader), &retry,
-			       sEntry->RetryCount, &sEntry->RInterval);
 	}
     }
 
@@ -778,35 +770,50 @@ static int CheckWorried(struct SFTP_Entry *sEntry)
     /* Check the packets from SendWorriedLimit to SendAckLimit, and
        see if we should be Worried about any of them.  */
 {
-    long i, rexmit;
+    long i;
     unsigned long now, then;
     RPC2_PacketBuffer *thePacket;
+    struct CEntry *ce;
+    uint32_t queued, rto;
+    struct timeval tv;
+
+    ce = rpc2_GetConn(sEntry->LocalHandle);
+    if (!ce) {
+	sEntry->SendWorriedLimit = sEntry->SendAckLimit;
+	return 1;
+    }
 
     if (sEntry->SendWorriedLimit < sEntry->SendLastContig)
 	sEntry->SendWorriedLimit = sEntry->SendLastContig;
-    
-    TVTOTS(&sEntry->RInterval, rexmit);
+
     TVTOTS(&sEntry->LastSS, now);
+    queued = (sEntry->PacketSize + sizeof(struct RPC2_PacketHeader)) *
+	     (sEntry->SendAckLimit - sEntry->SendLastContig);
+
     for (i = sEntry->SendAckLimit; i > sEntry->SendWorriedLimit; i--) {
+	queued -= sEntry->PacketSize + sizeof(struct RPC2_PacketHeader);
 	if (TESTBIT(sEntry->SendTheseBits, i - sEntry->SendLastContig))
 	    continue;
 
+	rpc2_RetryInterval(ce, 0, &tv, queued,sizeof(struct RPC2_PacketHeader));
+	rto = tv.tv_sec * 1000000 + tv.tv_usec;
+
 	/* check the timestamp and see if a timeout interval has
 	   occurred, if so let's start thinking about retransmitting
 	   the packet */
 	thePacket = sEntry->ThesePackets[PBUFF(i)];
-	if (thePacket) {
-	    then = ntohl(thePacket->Header.TimeStamp);
-	    if ((long)TSDELTA(now, then) > rexmit) {
-		say(4, SFTP_DebugLevel,
-		    "Worried packet %ld, sent %lu, (%lu msec ago)\n",
-		    i, then, (long)TSDELTA(now, then));
-		break;
-	    }
+	if (!thePacket) continue;
+
+	then = ntohl(thePacket->Header.TimeStamp);
+	if ((long)TSDELTA(now, then) > rto) {
+	    say(4, SFTP_DebugLevel,
+		"Worried packet %ld, sent %lu, (%lu msec ago)\n",
+		i, then, (long)TSDELTA(now, then));
+	    break;
 	}
     }
     sEntry->SendWorriedLimit = i;
-    say(/*9*/4, SFTP_DebugLevel, "LastContig = %ld, Worried = %ld, AckLimit = %ld, MostRecent = %ld\n",
+    say(/*9*/4, SFTP_DebugLevel, "LastContig = %d, Worried = %d, AckLimit = %d, MostRecent = %d\n",
 				  sEntry->SendLastContig, sEntry->SendWorriedLimit, sEntry->SendAckLimit, sEntry->SendMostRecent);
 
     return (sEntry->SendWorriedLimit > sEntry->SendLastContig);
@@ -1187,18 +1194,18 @@ int sftp_StartArrived(RPC2_PacketBuffer *pBuff, struct SFTP_Entry *sEntry)
 	    switch(sftpd->Tag)
 		{
 		case FILEBYNAME:
-		    say(0, SFTP_DebugLevel, "%s: ", sftpd->FileInfo.ByName.LocalFileName);
+		    say(1, SFTP_DebugLevel, "%s: ", sftpd->FileInfo.ByName.LocalFileName);
 		    break;
 
 		case FILEBYFD:
-		    say(0, SFTP_DebugLevel, "%ld: ", sftpd->FileInfo.ByFD.fd);
+		    say(1, SFTP_DebugLevel, "%ld: ", sftpd->FileInfo.ByFD.fd);
 		    break;
 
 		case FILEBYINODE:
-		    say(0, SFTP_DebugLevel, "%ld.%ld: ", sftpd->FileInfo.ByInode.Device, sftpd->FileInfo.ByInode.Inode);
+		    say(1, SFTP_DebugLevel, "%ld.%ld: ", sftpd->FileInfo.ByInode.Device, sftpd->FileInfo.ByInode.Inode);
 		    break;
 		case FILEINVM:
-		    say(0, SFTP_DebugLevel, "FILEINVM ");
+		    say(1, SFTP_DebugLevel, "FILEINVM ");
 		    break;
 		}
 	}
@@ -1261,20 +1268,22 @@ void sftp_InitPacket(RPC2_PacketBuffer *pb, struct SFTP_Entry *sfe,
 #ifdef RPC2DEBUG
 void PrintDb(struct SFTP_Entry *se, RPC2_PacketBuffer *pb)
 {
-    printf("SFTP_Entry:\n");
-    printf("\tMagic = %ld  WhoAmI = %d  LocalHandle = %#x  GotParms = %ld  SentParms = %ld\n",
+    fprintf(rpc2_tracefile, "SFTP_Entry:\n");
+    fprintf(rpc2_tracefile, "\tMagic = %ld  WhoAmI = %d  LocalHandle = %#x  GotParms = %d  SentParms = %d\n",
 	se->Magic, se->WhoAmI, se->LocalHandle, se->GotParms, se->SentParms);
-    printf("\topenfd = %ld  XferState = %ld  HitEOF = %ld  CtrlSeqNumber = %ld\n",
+    fprintf(rpc2_tracefile, "\topenfd = %ld  XferState = %d  HitEOF = %d  CtrlSeqNumber = %d\n",
     	se->openfd, se->XferState, se->HitEOF, se->CtrlSeqNumber);
-    printf("\tSendLastContig = %ld   SendMostRecent = %ld  SendAckLimit = %ld SendWorriedLimit = %ld  ReadAheadCount = %ld\n",
+    fprintf(rpc2_tracefile, "\tSendLastContig = %d   SendMostRecent = %d  SendAckLimit = %d SendWorriedLimit = %d  ReadAheadCount = %d\n",
 	se->SendLastContig,	se->SendMostRecent, se->SendAckLimit, se->SendWorriedLimit, se->ReadAheadCount);
-    printf("\tRecvLastContig = %ld   RecvMostRecent = %ld\n",
+    fprintf(rpc2_tracefile, "\tRecvLastContig = %d   RecvMostRecent = %d\n",
 	se->RecvLastContig,	se->RecvMostRecent);
 
     if (!pb) return;
 
-    printf("\nSFTP_Packet:\n");
+    fprintf(rpc2_tracefile, "\nSFTP_Packet:\n");
+    rpc2_htonp(pb);
     rpc2_PrintPacketHeader(pb, rpc2_tracefile);
+    rpc2_ntohp(pb);
 }
 #endif
 
