@@ -189,10 +189,26 @@ static int PacketCame(void)
     return rpc2_CheckFDs(IOMGR_Select, t ? &t->TimeLeft : NULL);
 }
 
+static void DispatchPacket(RPC2_PacketBuffer *pb)
+{
+    unsigned int i, ProtoVersion = ntohl(pb->Header.ProtoVersion);
+
+    for (i = 0; i < nPacketHandlers; i++) {
+	if (ProtoVersion == PacketHandlers[i].ProtoVersion) {
+	    PacketHandlers[i].Handler(pb);
+	    return;
+	}
+    }
+
+    /* we don't have a ghost of a chance */
+    BOGUS(pb, "Wrong version\n");
+}
+
 static void rpc2_ProcessPacket(int fd)
 {
     RPC2_PacketBuffer *pb = NULL;
-    unsigned int i, ProtoVersion;
+    struct timeval tv;
+    int rc;
 
     /* We are guaranteed that there is a packet in the socket
        buffer at this point */
@@ -225,16 +241,15 @@ static void rpc2_ProcessPacket(int fd)
 	return;
     }
 
-    ProtoVersion = ntohl(pb->Header.ProtoVersion);
-    for (i = 0; i < nPacketHandlers; i++) {
-	if (ProtoVersion == PacketHandlers[i].ProtoVersion) {
-	    PacketHandlers[i].Handler(pb);
-	    return;
-	}
+    rc = LUA_fail_delay(pb->Prefix.PeerAddr, pb, 0, &tv);
+    if (rc == -1) {
+	say(9, RPC2_DebugLevel, "Dropping incoming packet\n");
+	RPC2_FreeBuffer(&pb);
+	return;
     }
+    if (rc && rpc2_DelayedRecv(pb, &tv)) { return; } /* delay */
 
-    /* we don't have a ghost of a chance */
-    BOGUS(pb, "Wrong version\n");
+    DispatchPacket(pb);
 }
 
 void rpc2_SocketListener(void *dummy)
@@ -344,6 +359,7 @@ void rpc2_ExpireEvents()
 
 	for (i = TM_Rescan(rpc2_TimerQueue); i > 0; i--) {
 		t = TM_GetExpired(rpc2_TimerQueue);
+		if (!t) continue;
 		sl = (struct SL_Entry *)t->BackPointer;
 		rpc2_DeactivateSle(sl, TIMEOUT);
 
@@ -353,6 +369,13 @@ void rpc2_ExpireEvents()
 		else if (sl->Type == DELACK)
 		    DelayedAck(sl);
 
+		else if (sl->Type == DELAYED_SEND)
+		    rpc2_SendDelayedPacket(sl);
+
+		else if (sl->Type == DELAYED_RECV) {
+		    RPC2_PacketBuffer *pb = rpc2_RecvDelayedPacket(sl);
+		    DispatchPacket(pb);
+		}
 		else
 		    LWP_NoYieldSignal((char *)sl);
 	}
@@ -373,7 +396,7 @@ static void HandleSLPacket(RPC2_PacketBuffer *pb, struct CEntry *ce)
 	return;
     }
 
-    say(0, RPC2_DebugLevel, "HandleNak()\n");
+    say(1, RPC2_DebugLevel, "HandleNak()\n");
 
     rpc2_Recvd.Naks++;
 
@@ -508,7 +531,7 @@ static void DecodePacket(RPC2_PacketBuffer *pb, struct CEntry *ce)
 	case RPC2_INITMULTICAST: {
 		if (TestState(ce, SERVER, S_AWAITENABLE))
 		{
-			say(0, RPC2_DebugLevel, "Connection not enabled\n");
+			say(1, RPC2_DebugLevel, "Connection not enabled\n");
 			BOGUS(pb, "DecodePacket(INITMC): connection not enabled\n");
 			return;
 		}
@@ -540,7 +563,7 @@ static void DecodePacket(RPC2_PacketBuffer *pb, struct CEntry *ce)
 	/* cannot be any negative opcode XXXXXXXXXXXXXXXX */
 	default: {
 		if (TestState(ce, SERVER, S_AWAITENABLE)) {
-			say(0, RPC2_DebugLevel, "Connection not enabled\n");
+			say(1, RPC2_DebugLevel, "Connection not enabled\n");
 			BOGUS(pb, "DecodePacket: connection not enabled\n");
 			return;
 		}
@@ -681,19 +704,15 @@ static void HandleBusy(RPC2_PacketBuffer *pb,  struct CEntry *ce)
 {
 	struct SL_Entry *sl;
 
-	say(0, RPC2_DebugLevel, "HandleBusy(%x)\n", ce->UniqueCID);
+	say(1, RPC2_DebugLevel, "HandleBusy(%x)\n", ce->UniqueCID);
 
 	rpc2_Recvd.Busies++;
 	if (BogusSl(ce, pb))
 		return;
 
 	/* update rtt/bw measurements */
-	ce->respsize = pb->Prefix.LengthOfPacket;
 	rpc2_UpdateRTT(pb, ce);
 
-	/* now continue waiting for the real reply again */
-	ce->respsize = 0;
-
 	rpc2_Recvd.GoodBusies++;
 	sl = ce->MySl;
 	rpc2_DeactivateSle(sl, KEPTALIVE);
@@ -707,18 +726,17 @@ static void HandleCurrentReply(RPC2_PacketBuffer *pb, struct CEntry *ce)
 {
 	struct SL_Entry *sl;
 
-	say(0, RPC2_DebugLevel, "HandleCurrentReply()\n");
+	say(1, RPC2_DebugLevel, "HandleCurrentReply()\n");
 	rpc2_Recvd.Replies++;
 	/* should this assert ?? XXXX */
 	if (BogusSl(ce, pb))
 		return;
 
 	pb = ShrinkPacket(pb);
-	ce->respsize = pb->Prefix.LengthOfPacket;
 	rpc2_UpdateRTT(pb, ce);
 	rpc2_Recvd.GoodReplies++;
 	sl = ce->MySl;
-	sl->Packet = pb;
+	sl->data = pb;
 	SetState(ce, C_THINK);
 	rpc2_IncrementSeqNumber(ce);
 	rpc2_DeactivateSle(sl, ARRIVED);
@@ -731,7 +749,7 @@ static void HandleNewRequest(RPC2_PacketBuffer *pb, struct CEntry *ce)
 {
 	struct SL_Entry *sl;
 
-	say(0, RPC2_DebugLevel, "HandleNewRequest()\n");
+	say(1, RPC2_DebugLevel, "HandleNewRequest()\n");
 	pb = ShrinkPacket(pb);
 
 	ce->TimeStampEcho = pb->Header.TimeStamp;
@@ -753,10 +771,10 @@ static void HandleNewRequest(RPC2_PacketBuffer *pb, struct CEntry *ce)
 	rpc2_IncrementSeqNumber(ce);
 
 	{ /* set up a timer to send a unsolicited ack response (actually an
-	     RCP2_BUSY) within 200ms. */
+	     RCP2_BUSY) within 100ms. */
 	    struct timeval tv;
 	    tv.tv_sec = 0;
-	    tv.tv_usec = 200000;
+	    tv.tv_usec = RPC2_DELACK_DELAY;
 	    sl = rpc2_AllocSle(DELACK, ce);
 	    rpc2_ActivateSle(sl, &tv);
 	}
@@ -767,7 +785,7 @@ static void HandleNewRequest(RPC2_PacketBuffer *pb, struct CEntry *ce)
 		assert(sl->MagicNumber == OBJ_SLENTRY);
 		SetState(ce, S_PROCESS);
 		rpc2_DeactivateSle(sl, ARRIVED);
-		sl->Packet = pb;
+		sl->data = pb;
 		LWP_NoYieldSignal((char *)sl);
 	}  else	{
 		/* hold for a future RPC2_GetRequest() */
@@ -795,7 +813,7 @@ static struct SL_Entry *FindRecipient(RPC2_PacketBuffer *pb)
 
 static void HandleCurrentRequest(RPC2_PacketBuffer *pb, struct CEntry *ce)
 {
-	say(0, RPC2_DebugLevel, "HandleCurrentRequest()\n");
+	say(1, RPC2_DebugLevel, "HandleCurrentRequest()\n");
 
 	rpc2_Recvd.Requests++;
 
@@ -828,7 +846,7 @@ static void HandleInit1(RPC2_PacketBuffer *pb)
 	    return;
 	}
 
-	say(0, RPC2_DebugLevel, "HandleInit1()\n");
+	say(1, RPC2_DebugLevel, "HandleInit1()\n");
 
 	rpc2_Recvd.Requests++;
 
@@ -841,39 +859,56 @@ static void HandleInit1(RPC2_PacketBuffer *pb)
 	    return;
 	}
 
+	/* Find a willing LWP in RPC2_GetRequest() */
+	sl = FindRecipient(pb);
+	if (!sl) {
+	    /* no threads are ready, send a busy to the other side so that it
+	     * will back off and try again later */
+	    say(1, RPC2_DebugLevel,
+		"HandleInit1: No available threads, busying peer\n");
+
+	    /* SendBusy(ce, FALSE);
+	     * -- we don't have a CEntry yet, but we can reuse most of the
+	     * INIT1 packet we just got... */
+	    pb->Header.RemoteHandle ^= pb->Header.LocalHandle;
+	    pb->Header.LocalHandle ^= pb->Header.RemoteHandle;
+	    pb->Header.RemoteHandle ^= pb->Header.LocalHandle;
+	    pb->Header.Opcode = RPC2_BUSY;
+	    pb->Header.Lamport = RPC2_LamportTime();
+	    pb->Header.SeqNumber = -1;
+	    pb->Header.BodyLength = 0;
+	    pb->Prefix.LengthOfPacket = sizeof(struct RPC2_PacketHeader);
+
+	    rpc2_htonp(pb);
+
+	    rpc2_XmitPacket(pb, pb->Prefix.PeerAddr, 1);
+	    RPC2_FreeBuffer(&pb);
+	    return;
+	}
+
 	/* Create a connection entry */
 	ce = MakeConn(pb);
-	if (ce == NULL) {
-		/* Packet must have been bogus in some way */
-		BOGUS(pb, "HandleInit1: MakeConn failed\n");
-		return;
+	if (!ce) {
+	    /* Packet must have been bogus in some way */
+	    BOGUS(pb, "HandleInit1: MakeConn failed\n");
+	    return;
 	}
 
 	/* Now fix packet header so that it has a real RemoteHandle */
 	pb->Header.RemoteHandle = ce->UniqueCID;
 
-	/* we're going to keep this packet around, so try to save some space */
-	pb = ShrinkPacket(pb);
-
-	/* Find a willing LWP in RPC2_GetRequest() */
-	sl = FindRecipient(pb);
-	if (sl != NULL) {
-		assert(sl->MagicNumber == OBJ_SLENTRY);
-		rpc2_DeactivateSle(sl, ARRIVED);
-		sl->Packet = pb;
-		ce->Filter = sl->Filter; /* struct assignment */
-		ce->Filter.OldOrNew = OLDORNEW;
-		LWP_NoYieldSignal((char *)sl);
-	} else {
-		/* hold for a future RPC2_GetRequest() */
-		rpc2_HoldPacket(pb);
-	}
+	assert(sl->MagicNumber == OBJ_SLENTRY);
+	rpc2_DeactivateSle(sl, ARRIVED);
+	sl->data = pb;
+	ce->Filter = sl->Filter; /* struct assignment */
+	ce->Filter.OldOrNew = OLDORNEW;
+	LWP_NoYieldSignal((char *)sl);
 }
 
 
 static void HandleRetriedBind(RPC2_PacketBuffer *pb, struct CEntry *ce)
 {
-	say(0, RPC2_DebugLevel, "HandleRetriedBind()\n");
+	say(1, RPC2_DebugLevel, "HandleRetriedBind()\n");
 
 	if (!TestRole(ce, SERVER)) {
 		BOGUS(pb, "HandleRetriedBind: not server\n");
@@ -895,7 +930,7 @@ static void HandleRetriedBind(RPC2_PacketBuffer *pb, struct CEntry *ce)
 
 	if (TestState(ce, SERVER, S_STARTBIND)) {
 		/* Cases (1) and (2) */
-		say(0, RPC2_DebugLevel, "Busying Init1 on %#x\n", ce->UniqueCID);
+		say(1, RPC2_DebugLevel, "Busying Init1 on %#x\n", ce->UniqueCID);
 
 		SendBusy(ce, FALSE);
 		RPC2_FreeBuffer(&pb);
@@ -903,7 +938,7 @@ static void HandleRetriedBind(RPC2_PacketBuffer *pb, struct CEntry *ce)
 	}
 	if (ce->SecurityLevel == RPC2_OPENKIMONO && ce->HeldPacket) {
 		/* Case (3): The Init2 must have been dropped; resend it */
-		say(0, RPC2_DebugLevel, "Resending Init2 %#x\n",  ce->UniqueCID);
+		say(1, RPC2_DebugLevel, "Resending Init2 %#x\n",  ce->UniqueCID);
 		ce->HeldPacket->Header.TimeStamp = htonl(ce->TimeStampEcho);
 		rpc2_XmitPacket(ce->HeldPacket, ce->HostInfo->Addr, 1);
 		RPC2_FreeBuffer(&pb);
@@ -920,7 +955,7 @@ static void HandleInit2(RPC2_PacketBuffer *pb, struct CEntry *ce)
 {
 	struct SL_Entry *sl;
 
-	say(0, RPC2_DebugLevel, "HandleInit2()\n");
+	say(1, RPC2_DebugLevel, "HandleInit2()\n");
 
 	rpc2_Recvd.Requests++;
 
@@ -928,10 +963,9 @@ static void HandleInit2(RPC2_PacketBuffer *pb, struct CEntry *ce)
 		return;
 
 	pb = ShrinkPacket(pb);
-	ce->respsize = pb->Prefix.LengthOfPacket;
 	rpc2_UpdateRTT(pb, ce);
 	sl = ce->MySl;
-	sl->Packet = pb;
+	sl->data = pb;
 
 	/* we're only done if we have an old-style anonymous binding */
 	if (!pb->Prefix.sa && ce->SecurityLevel == RPC2_OPENKIMONO)
@@ -949,17 +983,16 @@ static void HandleInit4(pb, ce)
     {
     struct SL_Entry *sl;
 
-    say(0, RPC2_DebugLevel, "HandleInit4()\n");
+    say(1, RPC2_DebugLevel, "HandleInit4()\n");
 
     rpc2_Recvd.Requests++;
 
     if (BogusSl(ce, pb)) return;
     pb = ShrinkPacket(pb);
 
-    ce->respsize = pb->Prefix.LengthOfPacket;
     rpc2_UpdateRTT(pb, ce);
     sl = ce->MySl;
-    sl->Packet = pb;
+    sl->data = pb;
 
     SetState(ce, C_THINK);
     rpc2_DeactivateSle(sl, ARRIVED);
@@ -971,7 +1004,7 @@ static void HandleInit3(RPC2_PacketBuffer *pb, struct CEntry *ce)
 {
 	struct SL_Entry *sl;
 
-	say(0, RPC2_DebugLevel, "HandleInit3()\n");
+	say(1, RPC2_DebugLevel, "HandleInit3()\n");
 
 	rpc2_Recvd.Requests++;
 
@@ -982,7 +1015,7 @@ static void HandleInit3(RPC2_PacketBuffer *pb, struct CEntry *ce)
 			ce->HeldPacket->Header.TimeStamp = htonl(pb->Header.TimeStamp);
 			rpc2_XmitPacket(ce->HeldPacket, ce->HostInfo->Addr, 1);
 		}  else
-			say(0, RPC2_DebugLevel, "Bogus Init3\n");
+			say(1, RPC2_DebugLevel, "Bogus Init3\n");
 		/* Throw packet away anyway */
 		RPC2_FreeBuffer(&pb);
 		return;
@@ -999,7 +1032,7 @@ static void HandleInit3(RPC2_PacketBuffer *pb, struct CEntry *ce)
 	    ce->TimeStampEcho, ce->RequestTime);
 
 	sl = ce->MySl;
-	sl->Packet = pb;
+	sl->data = pb;
 	SetState(ce, S_FINISHBIND);
 	rpc2_DeactivateSle(sl, ARRIVED);
 	LWP_NoYieldSignal((char *)sl);
@@ -1015,7 +1048,7 @@ static void SendNak(RPC2_PacketBuffer *pb)
 	if (pb->Header.Opcode == RPC2_NAKED)
 	    return;
 
-	say(0, RPC2_DebugLevel, "Sending NAK\n");
+	say(1, RPC2_DebugLevel, "Sending NAK\n");
 	RPC2_AllocBuffer(0, &nakpb);
 	rpc2_InitPacket(nakpb, NULL, 0);
 	nakpb->Prefix.sa = pb->Prefix.sa;
@@ -1053,10 +1086,10 @@ static struct CEntry *MakeConn(struct RPC2_PacketBuffer *pb)
 
 	if (pb->Prefix.LengthOfPacket < INIT1LENGTH ||
 	    pb->Prefix.LengthOfPacket < (INIT1LENGTH +
-				     ntohl(ib1->FakeBody.ClientIdent.SeqLen)))
+				     ntohl(ib1->FakeBody_ClientIdent_SeqLen)))
 	{
 		/* avoid memory reference errors from bogus packets */
-		say(0, RPC2_DebugLevel, "Ignoring short Init1 packet\n");
+		say(1, RPC2_DebugLevel, "Ignoring short Init1 packet\n");
 		return NULL;
 	}
 
@@ -1077,7 +1110,7 @@ static struct CEntry *MakeConn(struct RPC2_PacketBuffer *pb)
 
 	if (ce->SecurityLevel != RPC2_OPENKIMONO) {
 	    secure_random_bytes(&ce->NextSeqNumber, sizeof(ce->NextSeqNumber));
-	    ce->EncryptionType = ntohl(ib1->FakeBody.EncryptionType);
+	    ce->EncryptionType = ntohl(ib1->FakeBody_EncryptionType);
 	}
 
 	SetRole(ce, SERVER);
@@ -1109,7 +1142,7 @@ void rpc2_IncrementSeqNumber(struct CEntry *ce)
 
 static void HandleOldRequest(RPC2_PacketBuffer *pb, struct CEntry *ce)
 {
-	say(0, RPC2_DebugLevel, "HandleOldRequest()\n");
+	say(1, RPC2_DebugLevel, "HandleOldRequest()\n");
 
 	rpc2_Recvd.Requests++;
 
