@@ -149,7 +149,7 @@ static void check_aes_monte_carlo(int verbose)
 	    }
 
 	    if (memcmp(ebuf, etestvector, AES_BLOCK_SIZE) != 0 ||
-	        memcmp(ebuf, etestvector, AES_BLOCK_SIZE) != 0)
+		memcmp(dbuf, dtestvector, AES_BLOCK_SIZE) != 0)
 	    {
 		fprintf(stderr, "AES monte carlo test FAILED\n");
 		exit(-1);
@@ -464,6 +464,8 @@ static void check_pbkdf_timing(int verbose)
     if (verbose)
 	fprintf(stderr, "Password Based Key Derivation:  ");
 
+    memset(key, 0, sizeof(key));
+    memset(salt, 0, sizeof(salt));
     memset(password, 0, sizeof(password));
 
     gettimeofday(&begin, NULL);
